#!/usr/bin/python
# -*- coding: utf-8 -*-

import asyncio
import argparse
from ..bcommon import utils
from .config import Config


def main():
    parser = argparse.ArgumentParser(description='Bubble server integrator',
                                     formatter_class=argparse.ArgumentDefaultsHelpFormatter)
    parser.add_argument('port', nargs='?', help='Port to listen', type=int, default=Config.Port)
    args = parser.parse_args()
    Config.set(args)
    loop = asyncio.get_event_loop()
    from .server import BubbleProtocol
    coro = loop.create_server(BubbleProtocol, '0.0.0.0', args.port)
    server = loop.run_until_complete(coro)
    print(f'Bubble server integrator {utils.get_version()}, '
          f'(c) Vadim Dyadkin 2014-2017, ESRF, inspired by Giuseppe Portale')
    print('If you use this program, please cite this paper: http://dx.doi.org/10.1107/S1600577516002411')
    print('Official web page: http://soft.snbl.eu/bubble.html')
    print('Mercurial repository: http://hg.3lp.cx/bubble')
    print(f'Mercurial hash: {utils.get_hg_hash()}')
    print('Serving on {}:{:d}'.format(*server.sockets[0].getsockname()))
    try:
        loop.run_forever()
    except KeyboardInterrupt:
        print('Ctrl-C has been pressed. Exit and clean up...')
    # it should be a 'finally' section here, but then we have to close the PeriodicTask properly.
    # It needs a refactoring before we put it in motion.
    server.close()
    loop.run_until_complete(server.wait_closed())
    loop.close()


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

import time
import json
import pickle
import struct
import asyncio
from ..bcommon.default import PICKLE_HEADER_STRUCT, PICKLE_MAGIC_NUMBER
from .watcher import Watcher


class BubbleWorker:
    def __init__(self):
        self.startedTimestamp = time.time()
        self.killed = False
        self._pickle = False
        self.runLocally = False
        self.waxsWatcher = Watcher('waxs')
        self.saxsWatcher = Watcher('saxs')
        self.pardict = {
            'kill': self.kill,
            'state': self.state,
            'saxs': self.saxs,
            'waxs': self.waxs,
            'pickle': self.pickle,
        }

    def pickle(self, request):
        self._pickle = request
        return {}

    def saxs(self, request):
        return self.setIntegrator(request, self.saxsWatcher)

    def waxs(self, request):
        return self.setIntegrator(request, self.waxsWatcher)

    # noinspection PyUnusedLocal
    def kill(self, request=None):
        loop = asyncio.get_event_loop()
        loop.call_soon(self.saxsWatcher.stopWatching)
        loop.call_soon(self.waxsWatcher.stopWatching)
        loop.call_later(0.5, loop.stop)
        self.killed = True
        return {}

    # noinspection PyUnusedLocal
    def state(self, request=None):
        return {
            'server': self.stateServer(),
            'saxs': self.stateSAXS(),
            'waxs': self.stateWAXS(),
        }

    def stateSAXS(self):
        state = self.saxsWatcher.state()
        if self.killed:
            state['running'] = False
        return state

    def stateWAXS(self):
        state = self.waxsWatcher.state()
        if self.killed:
            state['running'] = False
        return state

    def stateServer(self):
        return {
            'startedTimestamp': self.startedTimestamp,
            'killed': self.killed,
        }

    def setIntegrator(self, params, watcherObject):
        params['pickle'] = self._pickle
        watcherObject.setParameters(params)
        state = {}
        if watcherObject.errors:
            state['errors'] = watcherObject.errors
        if watcherObject.warnings:
            state['warnings'] = watcherObject.warnings
        return state

    def parse(self, request):
        state = {}
        for key in request:
            if key in self.pardict:
                state.update(self.pardict[key](request[key] if key in request else None))
        return self.pack_state(state)

    def pack_state(self, state):
        if self._pickle:
            packet = pickle.dumps(state)
            header = struct.pack(PICKLE_HEADER_STRUCT, PICKLE_MAGIC_NUMBER, len(packet))
            return header + packet
        else:
            return '{}\r\n'.format(json.dumps(state)).encode()

    def resetSent(self):
        self.waxsWatcher.resetSent()
        self.saxsWatcher.resetSent()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import datetime
import requests
from PyQt5 import QtCore, QtGui, QtWidgets
from ..bcommon.utils import get_hg_hash, get_version
from .fixedwidget import FixedWidget
from .ui.ui_wabout import Ui_WBubbleAbout


class WAboutBubble(QtWidgets.QDialog, Ui_WBubbleAbout, FixedWidget):
    def __init__(self, parent=None):
        super().__init__(parent=parent)
        self.setupUi(self)
        lt = self.aboutLabel.text()
        lt = lt.replace('@', get_hg_hash()).replace('#', get_version())
        self.aboutLabel.setText(lt)
        self.setWindowIcon(QtGui.QIcon(':/swiss'))
        self.fixWindow()

    @QtCore.pyqtSlot()
    def on_closeButton_clicked(self):
        self.close()

    @QtCore.pyqtSlot()
    def on_aboutQtButton_clicked(self):
        QtWidgets.QMessageBox.aboutQt(self)


class UpdateChecker(QtCore.QObject):
    Url = 'https://soft.snbl.eu/_sources/bubble.rst.txt'
    VersionToken = '.. Bubble Versions'
    sigNewVersion = QtCore.pyqtSignal(str, bool)
    sigFinished = QtCore.pyqtSignal()

    def check(self):
        try:
            r = requests.get(self.Url)
        except requests.RequestException as err:
            self.sigNewVersion.emit(str(err), True)
        else:
            self.parse(r.text)
        self.sigFinished.emit()

    def parse(self, text):
        version = get_version()
        strings = []
        put = False
        boldline = False
        first = True
        parsing = False
        for line in text.split('\n'):
            if not line:
                continue
            if line == self.VersionToken:
                parsing = True
                continue
            if parsing:
                if line.startswith('..'):
                    continue
                items = line.split()
                try:
                    newv = [int(i) for i in items[1].split('.')]
                except (IndexError, ValueError):
                    pass
                else:
                    try:
                        put = datetime.datetime(*newv) > datetime.datetime(*[int(i) for i in version.split('.')])
                    except ValueError:
                        pass
                    boldline = True
                if put:
                    line = line.strip()
                    line = line[line.index(" ") + 1:]
                    if boldline:
                        boldline = False
                        if not first:
                            strings.append('</ul>')
                        strings.append(f'<span style="font-weight: bold; color: red">{line}</span>'
                                       f'<ul style="list-style-type:disc">')
                    else:
                        strings.append(f'<li>{line}</li>')
                    first = False
        if strings:
            strings.append('</ul>')
        self.sigNewVersion.emit('\n'.join(strings), False)


class WUpdates(QtWidgets.QMessageBox):
    def __init__(self, parent):
        super().__init__(parent)
        self.setupUI()

    def setupUI(self):
        self.setWindowTitle('Bubble updates')
        self.setStandardButtons(self.Ok)
        self.cb = QtWidgets.QCheckBox('Do not check updates automatically')
        self.setCheckBox(self.cb)

    def error(self, text):
        self.setIcon(self.Critical)
        self.setText('There has been an error during update checking')
        self.setInformativeText(text)
        self.exec()

    def success(self, text):
        self.setIcon(self.Information)
        if text:
            self.setText('A new Bubble version is available at <a href=https://soft.snbl.eu/bubble.html#download>'
                         'soft.snbl.eu</a>')
            self.setInformativeText(text)
        else:
            self.setText('There are no new updates of Bubble')
        self.exec()



#!/usr/bin/python
# -*- coding: utf-8 -*-

import json
from PyQt5 import QtCore, QtWidgets
import pyqtgraph as pg
from pyqtgraph import dockarea


class WPlot(QtWidgets.QDialog):
    sigClosed = QtCore.pyqtSignal()

    def __init__(self, name, parent=None):
        super().__init__(parent=parent)
        self.data = None
        self.array = None
        self.name = name
        self.title = ''
        self.setupUI()
        self.setWindowTitle(f'{name} plot')

    def setupUI(self):
        self.area = dockarea.DockArea()
        d1 = dockarea.Dock('2D')
        d2 = dockarea.Dock('1D')
        self.area.addDock(d1, 'top')
        self.area.addDock(d2, 'bottom')
        self.plot2DView = pg.ImageView()
        self.plot1DView = pg.PlotWidget()
        d1.addWidget(self.plot2DView)
        d2.addWidget(self.plot1DView)
        self.plotItem = self.plot1DView.getPlotItem()
        self.imageItem = self.plot2DView.getImageItem()
        self.imageView = self.plot2DView.getView()
        self.vLine = pg.InfiniteLine(angle=90, movable=False)
        self.hLine = pg.InfiniteLine(angle=0, movable=False)
        self.plotItem.vb.addItem(self.vLine, ignoreBounds=True)
        self.plotItem.vb.addItem(self.hLine, ignoreBounds=True)
        layout = QtWidgets.QVBoxLayout()
        layout.addWidget(self.area)
        self.setLayout(layout)
        self.proxy1D = pg.SignalProxy(self.plotItem.scene().sigMouseMoved, rateLimit=60, slot=self.mouseMoved1D)

    def mouseMoved1D(self, evt):
        pos = evt[0]
        if self.plotItem.sceneBoundingRect().contains(pos):
            mousePoint = self.plotItem.vb.mapSceneToView(pos)
            x, y = mousePoint.x(), mousePoint.y()
            self.vLine.setPos(x)
            self.hLine.setPos(y)
            if self.data is None:
                return
            index = int(x)
            if 0 < index < len(self.data):
                self.plot1DView.setTitle(f'{self.title}; x={x:0.3f}, y={self.data[index]:0.3f}')

    def closeEvent(self, event):
        self.sigClosed.emit()

    def saveSettings(self):
        s = QtCore.QSettings()
        s.setValue(f'WPlot{self.name}/Geometry', self.saveGeometry())
        s.setValue(f'WPlot{self.name}/dockState', json.dumps(self.area.saveState()))

    def loadSettings(self):
        s = QtCore.QSettings()
        self.restoreGeometry(s.value(f'WPlot{self.name}/Geometry', b''))
        dockState = s.value(f'WPlot{self.name}/dockState', '', str)
        if dockState:
            # noinspection PyBroadException
            try:
                self.area.restoreState(json.loads(dockState))
            except Exception:
                pass

    def plot(self, response):
        data1d = response.get('data1d', None)
        if data1d:
            x, y, e = data1d
            try:
                y[y <= 0] = y[y > 0].min()
            except ValueError:  # it seems that the array is zero-like, skip it
                pass
            self.data = y
            self.plot1DView.plot(x, y, pen='g', clear=True)
            self.title = f'{response.get("chiFile", "")}; Tr = {response.get("transmission", 0):.5f}'
            self.plot1DView.setTitle(self.title)
        image = response.get('image', None)
        if image is not None:
            self.array = image
            self.plot2DView.setImage(self.array)
            self.plot2DView.setLevels(0, self.array.mean() + 8 * self.array.std())



#!/usr/bin/python
# -*- coding: utf-8 -*-

import os
from PyQt5 import QtGui, QtCore, QtWidgets
from ..bcommon import default
from . import connector, wplot, wsaxsparams, wmask, wdark, wabout, wsopts
from .fixedwidget import FixedWidget, clearButtonSlot
from .ui.wbubble import Ui_WBubble


class WBubble(QtWidgets.QDialog, Ui_WBubble, FixedWidget):
    killServerSignal = QtCore.pyqtSignal()
    startServerSignal = QtCore.pyqtSignal(str, int)
    runSignal = QtCore.pyqtSignal(dict, str)
    beamlineToggledSignal = QtCore.pyqtSignal(str)
    applyWaxsDarkSignal = QtCore.pyqtSignal()
    applySaxsDarkSignal = QtCore.pyqtSignal()
    MaxLabelLen = 40

    def __init__(self):
        super().__init__()
        self.saveFolder = ''
        self.threadUpdateChecker = None
        self.setupUi(self)
        self.setValidators()
        self.setVisibleWidgets()
        self.createWindows()
        self.createConnector()
        self.deleteThreadUpdateChecker()
        self.connectSignals()
        self.fixWindow()

    def createConnector(self):
        self.connector = connector.Connector()

    def setValidators(self):
        self.multiplyLineEdit.setValidator(QtGui.QDoubleValidator())

    def createWindows(self):
        self.wabout = wabout.WAboutBubble(self)
        self.wSaxsMask = wmask.WMask('saxs', self)
        self.wWaxsMask = wmask.WMask('waxs', self)
        self.waxsPlot = wplot.WPlot('WAXS', self)
        self.saxsPlot = wplot.WPlot('SAXS', self)
        self.saxsPlot.plot1DView.setLogMode(x=True, y=True)
        self.wSaxsParams = wsaxsparams.WSaxsParams(self)
        self.wSaxsDark = wdark.WDark(self)
        self.wWaxsDark = wdark.WDark(self)
        self.wsopts = wsopts.WSOpts(self)
        self.wupdates = wabout.WUpdates(self)

    def start(self):
        self.loadSettings()
        self.wSaxsMask.loadSettings()
        self.wWaxsMask.loadSettings()
        self.waxsPlot.loadSettings()
        self.saxsPlot.loadSettings()
        self.wSaxsParams.loadSettings()
        self.wSaxsDark.loadSettings()
        self.wWaxsDark.loadSettings()
        self.wsopts.loadSettings()
        self.startServerSignal.emit(self.hostLineEdit.text(), self.portSpinBox.value())
        self.show()
        self.wSaxsParams.showEvent(None)
        self.on_runSaxsButton_clicked(0)
        self.on_runWaxsButton_clicked(0)
        self.checkNewVersion()

    def setVisibleWidgets(self):
        self.stopServerButton.hide()
        self.runServerButton.show()
        self.killServerButton.hide()
        self.stopWaxsButton.hide()
        self.stopSaxsButton.hide()
        self.sspeedCheckBox.setEnabled(False)
        self.multiEdit.setDisabled(True)

    # noinspection PyUnresolvedReferences
    def connectSignals(self):
        self.connector.serverStateSignal.connect(self.showServerState)
        self.connector.errorsSignal.connect(self.showServerErrors)
        self.connector.warningsSignal.connect(self.showServerWarnings)
        self.killServerSignal.connect(self.connector.killServer)
        self.connector.waxsStateSignal.connect(self.showWaxsState)
        self.connector.saxsStateSignal.connect(self.showSaxsState)
        self.connector.waxsPlotSignal.connect(self.waxsPlot.plot)
        self.connector.saxsPlotSignal.connect(self.saxsPlot.plot)
        self.runSignal.connect(self.connector.runSaxsWaxs)
        self.startServerSignal.connect(self.connector.tryServer)
        self.beamlineToggledSignal.connect(self.wWaxsMask.changeBeamline)
        self.beamlineToggledSignal.connect(self.wSaxsMask.changeBeamline)
        self.beamlineToggledSignal.connect(self.connector.setBeamline)
        self.applySaxsDarkSignal.connect(self.wSaxsDark.on_buttonBox_accepted)
        self.applyWaxsDarkSignal.connect(self.wWaxsDark.on_buttonBox_accepted)
        self.saxsPlot.sigClosed.connect(lambda: self.saxsPlotButton.setChecked(False))
        self.waxsPlot.sigClosed.connect(lambda: self.waxsPlotButton.setChecked(False))
        self.wSaxsMask.sigMask.connect(self.saxsMaskLineEdit.setText)
        self.wWaxsMask.sigMask.connect(self.waxsMaskLineEdit.setText)
        self.wabout.buttonUpdate.clicked.connect(self.checkNewVersionByUser)

    def closeEvent(self, event):
        self.connector.disconnectFromServer()
        self.waxsPlot.close()
        self.saxsPlot.close()
        self.wSaxsMask.close()
        self.wWaxsMask.close()
        self.saveSettings()
        self.wsopts.saveSettings()
        self.wSaxsDark.saveSettings()
        self.wWaxsDark.saveSettings()
        self.waxsPlot.saveSettings()
        self.saxsPlot.saveSettings()
        self.wSaxsParams.saveSettings()
        self.deleteThreadUpdateChecker()

    def saveSettings(self, s=None):
        s = s or QtCore.QSettings()
        s.setValue('WBubble/Geometry', self.saveGeometry())
        s.setValue('WBubble/host', self.hostLineEdit.text())
        s.setValue('WBubble/port', self.portSpinBox.value())
        s.setValue('WBubble/speed', self.speedCheckBox.isChecked())
        s.setValue('WBubble/sspeed', self.sspeedCheckBox.isChecked())
        s.setValue('WBubble/waxspath', self.waxsDirLineEdit.text())
        s.setValue('WBubble/waxsPoniPath', self.waxsPoniLineEdit.text())
        s.setValue('WBubble/waxsBkgPath', self.waxsBkgLineEdit.text())
        s.setValue('WBubble/waxsBkgCoef', self.waxsBkgCoefSpinBox.value())
        s.setValue('WBubble/saxspath', self.saxsDirLineEdit.text())
        s.setValue('WBubble/saxsPoniPath', self.saxsPoniLineEdit.text())
        s.setValue('WBubble/saxsMaskPath', self.saxsMaskLineEdit.text())
        s.setValue('WBubble/waxsMaskPath', self.waxsMaskLineEdit.text())
        s.setValue('WBubble/saxsBkgPath', self.saxsBkgLineEdit.text())
        s.setValue('WBubble/saxsBkgCoef', self.saxsBkgCoefSpinBox.value())
        s.setValue('WBubble/saxsAziStart', self.saxsAziStartSpinBox.value())
        s.setValue('WBubble/saxsAziStop', self.saxsAziStopSpinBox.value())
        s.setValue('WBubble/multiply', self.multiplyLineEdit.text())
        s.setValue('WBubble/beamline', self.getBeamlineRadio())
        s.setValue('WBubble/outUnits', self.getUnitsRadio())
        s.setValue('WBubble/waxsNormalization', self.comboNorm.currentText())
        s.setValue('WBubble/extCheckbox', self.extCheckBox.isChecked())
        s.setValue('WBubble/saxsDetector', self.saxsDetectorComboBox.currentText())
        s.setValue('WBubble/waxsDetector', self.waxsDetectorComboBox.currentText())
        s.setValue('WBubble/subdirCheckbox', self.subdirCheckBox.isChecked())
        s.setValue('WBubble/ext', self.extEdit.text())
        s.setValue('WBubble/subdir', self.subdirEdit.text())
        s.setValue('WBubble/saveFolder', self.saveFolder)
        s.setValue('WBubble/multicolumn', self.multiCheckBox.isChecked())
        s.setValue('WBubble/multicolumnName', self.multiEdit.text())
        s.setValue('WBubble/saveEdf', self.edfCheckBox.isChecked())
        s.setValue('WBubble/waxsAzimuth', self.waxsAzimuthCheckBox.isChecked())
        s.setValue('WBubble/waxsAzimuthStart', self.waxsAziStartSpinBox.value())
        s.setValue('WBubble/waxsAzimuthStop', self.waxsAziStopSpinBox.value())
        s.setValue('WBubble/waxsAzimuthSlices', self.waxsSlicesCheckBox.isChecked())
        s.setValue('WBubble/saxsAzimuth', self.saxsAzimuthCheckBox.isChecked())
        s.setValue('WBubble/saxsAzimuthSlices', self.saxsSlicesCheckBox.isChecked())
        s.setValue('WBubble/waxsRadial', self.waxsRadialCheckBox.isChecked())
        s.setValue('WBubble/waxsRadialStart', self.waxsRadStartSpinBox.value())
        s.setValue('WBubble/waxsRadialStop', self.waxsRadStopSpinBox.value())
        s.setValue('WBubble/saxsRadialStart', self.saxsRadStartSpinBox.value())
        s.setValue('WBubble/saxsRadialStop', self.saxsRadStopSpinBox.value())
        s.setValue('WBubble/saxsRadial', self.saxsRadialCheckBox.isChecked())
        s.setValue('WBubble/page', self.tabWidget.currentIndex())
        s.setValue('WBubble/doNotCheckUpdates', self.wupdates.cb.isChecked())

    def loadSettings(self, s=None):
        s = s or QtCore.QSettings()
        self.restoreGeometry(s.value('WBubble/Geometry', b''))
        self.hostLineEdit.setText(s.value('WBubble/host', default.DEFAULT_HOST, str))
        self.portSpinBox.setValue(s.value('WBubble/port', default.DEFAULT_PORT, int))
        self.speedCheckBox.setChecked(s.value('WBubble/speed', False, bool))
        self.sspeedCheckBox.setChecked(s.value('WBubble/sspeed', False, bool))
        self.waxsDirLineEdit.setText(s.value('WBubble/waxspath', '', str))
        self.waxsPoniLineEdit.setText(s.value('WBubble/waxsPoniPath', '', str))
        self.waxsBkgLineEdit.setText(s.value('WBubble/waxsBkgPath', '', str))
        self.waxsBkgCoefSpinBox.setValue(s.value('WBubble/waxsBkgCoef', 0, float))
        self.saxsDirLineEdit.setText(s.value('WBubble/saxspath', '', str))
        self.saxsPoniLineEdit.setText(s.value('WBubble/saxsPoniPath', '', str))
        self.saxsMaskLineEdit.setText(s.value('WBubble/saxsMaskPath', '', str))
        self.waxsMaskLineEdit.setText(s.value('WBubble/waxsMaskPath', '', str))
        self.saxsBkgLineEdit.setText(s.value('WBubble/saxsBkgPath', '', str))
        self.saxsBkgCoefSpinBox.setValue(s.value('WBubble/saxsBkgCoef', 0, float))
        self.saxsAziStartSpinBox.setValue(s.value('WBubble/saxsAziStart', 0, float))
        self.saxsAziStopSpinBox.setValue(s.value('WBubble/saxsAziStop', 0, float))
        self.multiplyLineEdit.setText(s.value('WBubble/multiply', '1', str))
        beamline = s.value('WBubble/beamline', 'Dubble', str).replace('&', '')
        if beamline == 'SNBL':
            self.snblRadio.setChecked(True)
        elif beamline == 'Dubble':
            self.dubbleRadio.setChecked(True)
        norm = s.value('WBubble/waxsNormalization', 'Monitor', str)
        self.comboNorm.setCurrentIndex(self.comboNorm.findText(norm))
        self.saxsDetectorComboBox.setCurrentIndex(self.saxsDetectorComboBox.findText(
            s.value('WBubble/saxsDetector', 'Pilatus', str)))
        self.waxsDetectorComboBox.setCurrentIndex(
            self.waxsDetectorComboBox.findText(s.value('WBubble/waxsDetector', 'Pilatus', str)))
        outUnits = s.value('WBubble/outUnits', 'qRadio', str)
        if outUnits == 'tthRadio':
            self.tthRadio.setChecked(True)
        self.extCheckBox.setChecked(s.value('WBubble/extCheckbox', False, bool))
        self.subdirCheckBox.setChecked(s.value('WBubble/subdirCheckbox', False, bool))
        self.extEdit.setText(s.value('WBubble/ext', '', str))
        self.subdirEdit.setText(s.value('WBubble/subdir', '', str))
        self.saveFolder = s.value('WBubble/saveFolder', '', str)
        self.multiCheckBox.setChecked(s.value('WBubble/multicolumn', False, bool))
        self.multiEdit.setText(s.value('WBubble/multicolumnName', default.MULTICOLUMN_NAME, str))
        self.edfCheckBox.setChecked(s.value('WBubble/saveEdf', False, bool))
        self.waxsAziStartSpinBox.setValue(s.value('WBubble/waxsAzimuthStart', 0, float))
        self.waxsAziStopSpinBox.setValue(s.value('WBubble/waxsAzimuthStop', 0, float))
        self.waxsSlicesCheckBox.setChecked(s.value('WBubble/waxsAzimuthSlices', False, bool))
        self.saxsSlicesCheckBox.setChecked(s.value('WBubble/saxsAzimuthSlices', False, bool))
        self.saxsAzimuthCheckBox.setChecked(s.value('WBubble/saxsAzimuth', False, bool))
        self.saxsRadialCheckBox.setChecked(s.value('WBubble/saxsRadial', False, bool))
        self.waxsRadialCheckBox.setChecked(s.value('WBubble/waxsRadial', False, bool))
        self.waxsRadStartSpinBox.setValue(s.value('WBubble/waxsRadialStart', 0, float))
        self.waxsRadStopSpinBox.setValue(s.value('WBubble/waxsRadialStop', 0, float))
        self.saxsRadStartSpinBox.setValue(s.value('WBubble/saxsRadialStart', 0, float))
        self.saxsRadStopSpinBox.setValue(s.value('WBubble/saxsRadialStop', 0, float))
        self.tabWidget.setCurrentIndex(s.value('WBubble/page', 0, int))
        self.wupdates.cb.setChecked(s.value('WBubble/doNotCheckUpdates', False, bool))

    @QtCore.pyqtSlot()
    def on_exitButton_clicked(self):
        self.close()

    def showServerState(self, status):
        if status['serverRunning']:
            self.serverLampLabel.setPixmap(QtGui.QPixmap(':/grnbtn'))
            self.serverStateLabel.setText(status['serverRunning'])
            self.tabWidget.setTabIcon(0, QtGui.QIcon(QtGui.QPixmap(':/grnbtn')))
            self.stopServerButton.show()
            self.runServerButton.hide()
            self.killServerButton.show()
        else:
            self.serverLampLabel.setPixmap(QtGui.QPixmap(':/redbtn'))
            self.serverStateLabel.setText(status['serverRunning'])
            self.tabWidget.setTabIcon(0, QtGui.QIcon(QtGui.QPixmap(':/redbtn')))
            self.stopServerButton.hide()
            self.runServerButton.show()
            self.serverStateLabel.setText('Server is stopped')
            self.killServerButton.hide()

    def showWaxsState(self, status):
        running = status.get('running', False)
        if running:
            self.waxsLampLabel.setPixmap(QtGui.QPixmap(':/grnbtn'))
            self.tabWidget.setTabIcon(2, QtGui.QIcon(QtGui.QPixmap(':/grnbtn')))
            self.formatLabelText(status.get('path', 'unknown'), self.waxsStateLabel)
            self.runWaxsButton.hide()
            self.stopWaxsButton.show()
        else:
            self.waxsLampLabel.setPixmap(QtGui.QPixmap(':/redbtn'))
            self.tabWidget.setTabIcon(2, QtGui.QIcon(QtGui.QPixmap(':/redbtn')))
            self.formatLabelText('stopped', self.waxsStateLabel)
            self.stopWaxsButton.hide()
            self.runWaxsButton.show()
        self.formatLabelText(os.path.basename(status.get('imageFile', '')), self.waxsLastfileLabel)
        self.formatLabelText(f'{status.get("total", 0)} of {status.get("all", 0)}', self.waxsTotalLabel)

    def formatLabelText(self, text: str, label: QtWidgets.QLabel):
        if len(text) >= self.MaxLabelLen:
            label.setText(f'...{text[-self.MaxLabelLen:]}')
        else:
            label.setText(text)
        label.setToolTip(text)

    def showSaxsState(self, status):
        running = status.get('running', False)
        if running:
            self.saxsLampLabel.setPixmap(QtGui.QPixmap(':/grnbtn'))
            self.tabWidget.setTabIcon(1, QtGui.QIcon(QtGui.QPixmap(':/grnbtn')))
            self.formatLabelText(status.get('path', 'unknown'), self.saxsStateLabel)
            self.runSaxsButton.hide()
            self.stopSaxsButton.show()
        else:
            self.saxsLampLabel.setPixmap(QtGui.QPixmap(':/redbtn'))
            self.tabWidget.setTabIcon(1, QtGui.QIcon(QtGui.QPixmap(':/redbtn')))
            self.formatLabelText('stopped', self.saxsStateLabel)
            self.stopSaxsButton.hide()
            self.runSaxsButton.show()
        self.formatLabelText(os.path.basename(status.get('imageFile', '')), self.saxsLastfileLabel)
        self.formatLabelText(f'{status.get("total", 0)} of {status.get("all", 0)}', self.saxsTotalLabel)

    def showServerErrors(self, errors):
        QtWidgets.QMessageBox.critical(self, 'Server errors', errors)

    def showServerWarnings(self, warnings):
        QtWidgets.QMessageBox.warning(self, 'Server warnings', warnings)

    @QtCore.pyqtSlot(bool)
    def on_localCheckBox_toggled(self, checked):
        if checked:
            self.hostLineEdit.setDisabled(True)
            self.portSpinBox.setDisabled(True)
            self.runServerButton.setText('Start server')
            self.stopServerButton.setText('Stop server')
        else:
            self.hostLineEdit.setEnabled(True)
            self.portSpinBox.setEnabled(True)
            self.runServerButton.setText('Connect')
            self.stopServerButton.setText('Disconnect')

    @QtCore.pyqtSlot()
    def on_stopServerButton_clicked(self):
        self.stopServerButton.hide()
        self.runServerButton.show()
        self.killServerButton.hide()
        self.serverLampLabel.setPixmap(QtGui.QPixmap(':/redbtn'))
        self.serverStateLabel.setText('Disconnected from the server')
        self.connector.disconnectFromServer()

    @QtCore.pyqtSlot()
    def on_runServerButton_clicked(self):
        self.runServerButton.hide()
        self.runServerButton.show()
        self.startServerSignal.emit(self.hostLineEdit.text(), self.portSpinBox.value())

    @QtCore.pyqtSlot()
    def on_killServerButton_clicked(self):
        self.killServerSignal.emit()

    def getBeamlineRadio(self):
        return self.getRadio('Dubble', (self.dubbleRadio, self.snblRadio))

    def getUnitsRadio(self):
        return self.getRadio('q', (self.qRadio, self.tthRadio), False)

    def getRadio(self, start, radios, text=True):
        value = start
        for radio in radios:
            if radio.isChecked():
                if text:
                    value = radio.text().replace('&', '')
                else:
                    value = radio.objectName()
                break
        return value

    def getSpeed(self):
        return {
            'speed': self.speedCheckBox.isChecked(),
            'sspeed': self.sspeedCheckBox.isEnabled() and self.sspeedCheckBox.isChecked(),
        }

    def getCommonParams(self):
        params = {
            'beamline': self.getBeamlineRadio(),
            'units': self.getUnitsRadio()[0],
        }
        params.update(self.getSpeed())
        params.update(self.wsopts.params)
        return params

    @QtCore.pyqtSlot()
    def on_runWaxsButton_clicked(self, run=1):
        path = self.waxsDirLineEdit.text()
        poniPath = self.waxsPoniLineEdit.text()
        ext = self.extEdit.text() if self.extCheckBox.isChecked() else ''
        subdir = self.subdirEdit.text() if self.subdirCheckBox.isChecked() else ''
        multi = ''
        if self.multiCheckBox.isChecked():
            name = self.multiEdit.text()
            multi = name if name else default.MULTICOLUMN_NAME
        if run and (not path or not poniPath):
            QtWidgets.QMessageBox.critical(self, 'WAXS Error', 'You have to set folder and poni file, at least.')
            return
        multiplier = self.multiplyLineEdit.text()
        radial = None
        if self.waxsRadialCheckBox.isChecked():
            radial = self.waxsRadStartSpinBox.value(), self.waxsRadStopSpinBox.value()

        norm = []
        for n in (self.spinWaxsBkgStart.value(), self.spinWaxsBkgStop.value()):
            try:
                norm.append(float(n))
            except ValueError:
                norm.append(0)

        params = {
            'path': path,
            'run': run,
            'poni': poniPath,
            'bkgCoef': self.waxsBkgCoefSpinBox.value(),
            'backgroundFiles': [s.strip() for s in self.waxsBkgLineEdit.text().split(';') if s],
            'maskFile': self.waxsMaskLineEdit.text(),
            'calibration': float(multiplier) if multiplier else 1,
            'ext': ext,
            'subdir': subdir,
            'multicolumnName': multi,
            'save': self.edfCheckBox.isChecked(),
            'detector': self.waxsDetectorComboBox.currentText(),
            'azimuth': (self.waxsAziStartSpinBox.value(), self.waxsAziStopSpinBox.value()),
            'azimuthSlices': self.waxsSlicesCheckBox.isChecked(),
            'azimuthChecked': self.waxsAzimuthCheckBox.isChecked(),
            'radial': radial,
            'normalization': self.comboNorm.currentText(),
            'normrange': norm,
        }
        params.update(self.getCommonParams())
        params.update(self.wWaxsDark.params)
        self.runSignal.emit(params, 'waxs')

    @QtCore.pyqtSlot()
    def on_runSaxsButton_clicked(self, run=1):
        path = self.saxsDirLineEdit.text()
        poniPath = self.saxsPoniLineEdit.text()
        ext = str(self.extEdit.text()) if self.extCheckBox.isChecked() else ''
        subdir = str(self.subdirEdit.text()) if self.subdirCheckBox.isChecked() else ''
        multi = ''
        if self.multiCheckBox.isChecked():
            name = str(self.multiEdit.text())
            multi = name if name else default.MULTICOLUMN_NAME
        if run and (not path or not poniPath):
            QtWidgets.QMessageBox.critical(self, 'SAXS Error', 'You have to set folder and poni file, at least.')
            return
        radial = None
        if self.saxsRadialCheckBox.isChecked():
            radial = self.saxsRadStartSpinBox.value(), self.saxsRadStopSpinBox.value()
        params = {
            'path': path,
            'run': run,
            'poni': poniPath,
            'bkgCoef': self.saxsBkgCoefSpinBox.value(),
            'azimuth': (self.saxsAziStartSpinBox.value(), self.saxsAziStopSpinBox.value()),
            'maskFile': str(self.saxsMaskLineEdit.text()),
            'backgroundFiles': [s.strip() for s in str(self.saxsBkgLineEdit.text()).split(';') if s],
            'ext': ext,
            'subdir': subdir,
            'multicolumnName': multi,
            'save': self.edfCheckBox.isChecked(),
            'detector': self.saxsDetectorComboBox.currentText(),
            'azimuthSlices': self.saxsSlicesCheckBox.isChecked(),
            'azimuthChecked': self.saxsAzimuthCheckBox.isChecked(),
            'radial': radial,
            'normalization': 'Monitor',
            'normrange': (None, None),
        }
        params.update(self.getCommonParams())
        params.update(self.wSaxsParams.params)
        params.update(self.wSaxsDark.params)
        self.runSignal.emit(params, 'saxs')

    @QtCore.pyqtSlot()
    def on_stopWaxsButton_clicked(self):
        self.runSignal.emit({'stop': 1}, 'waxs')

    @QtCore.pyqtSlot()
    def on_stopSaxsButton_clicked(self):
        self.runSignal.emit({'stop': 1}, 'saxs')

    @QtCore.pyqtSlot(bool)
    def on_waxsPlotButton_toggled(self, checked):
        self.waxsPlot.setVisible(checked)

    @QtCore.pyqtSlot(bool)
    def on_saxsPlotButton_toggled(self, checked):
        self.saxsPlot.setVisible(checked)

    @QtCore.pyqtSlot()
    def on_waxsFolderButton_clicked(self):
        self._folderButton_clicked(self.waxsDirLineEdit)

    @QtCore.pyqtSlot()
    def on_saxsFolderButton_clicked(self):
        self._folderButton_clicked(self.saxsDirLineEdit)

    def _folderButton_clicked(self, dirLineEdit):
        d = QtWidgets.QFileDialog.getExistingDirectory(self, 'Select folder', dirLineEdit.text())
        if d:
            dirLineEdit.setText(d)

    @QtCore.pyqtSlot()
    def on_waxsPoniButton_clicked(self):
        self._poniButton_clicked(self.waxsPoniLineEdit)

    @QtCore.pyqtSlot()
    def on_saxsPoniButton_clicked(self):
        self._poniButton_clicked(self.saxsPoniLineEdit)

    def _poniButton_clicked(self, poniLineEdit):
        currentPoni = poniLineEdit.text()
        if currentPoni:
            currentPoni = os.path.dirname(currentPoni)
        poni = QtWidgets.QFileDialog.getOpenFileName(self, 'Select poni file', currentPoni, 'Poni (*.poni)')[0]
        if poni:
            poniLineEdit.setText(poni)

    @QtCore.pyqtSlot()
    def on_waxsBkgButton_clicked(self):
        self._bkgButton_clicked(self.waxsBkgLineEdit)

    @QtCore.pyqtSlot()
    def on_saxsBkgButton_clicked(self):
        self._bkgButton_clicked(self.saxsBkgLineEdit)

    def _bkgButton_clicked(self, bkgLineEdit):
        currentBkg = bkgLineEdit.text()
        if currentBkg:
            currentBkg = os.path.dirname(currentBkg.split(';')[0])
        bkg = QtWidgets.QFileDialog.getOpenFileNames(self, 'Select background files', currentBkg,
                                                     'Frames (*.edf *.cbf)')[0]
        if bkg:
            bkgLineEdit.setText(';'.join(bkg))

    @QtCore.pyqtSlot()
    def on_saxsMaskButton_clicked(self):
        self._maskButton_clicked(self.saxsMaskLineEdit)

    @QtCore.pyqtSlot()
    def on_waxsMaskButton_clicked(self):
        self._maskButton_clicked(self.waxsMaskLineEdit)

    def _maskButton_clicked(self, maskLineEdit):
        mask = maskLineEdit.text()
        mask = QtWidgets.QFileDialog.getOpenFileName(self, 'Select mask file', mask,
                                                     'Masks (*.msk *.npz);;Fit2D mask (*.msk);;Bubble mask (*.npz)')[0]
        if mask:
            maskLineEdit.setText(mask)

    @QtCore.pyqtSlot()
    def on_advancedButton_clicked(self):
        self.wSaxsParams.exec()

    @QtCore.pyqtSlot(bool)
    def on_snblRadio_toggled(self, checked):
        if checked:
            self.beamlineToggledSignal.emit('SNBL')
            self.tabWidget.setTabEnabled(1, False)

    @QtCore.pyqtSlot(bool)
    def on_dubbleRadio_toggled(self, checked):
        if checked:
            self.beamlineToggledSignal.emit('Dubble')
            self.tabWidget.setTabEnabled(1, True)

    @QtCore.pyqtSlot(bool)
    def on_extCheckBox_toggled(self, checked):
        self.extEdit.setEnabled(checked)

    @QtCore.pyqtSlot()
    def on_darkFilesButton_clicked(self):
        self.wSaxsDark.exec()

    @QtCore.pyqtSlot()
    def on_waxsDarkFilesButton_clicked(self):
        self.wWaxsDark.exec()

    @QtCore.pyqtSlot(bool)
    def on_subdirCheckBox_toggled(self, checked):
        self.subdirEdit.setEnabled(checked)

    @QtCore.pyqtSlot(bool)
    def on_multiCheckBox_toggled(self, checked):
        self.multiEdit.setEnabled(checked)

    @QtCore.pyqtSlot()
    def on_saveButton_clicked(self):
        fn = QtWidgets.QFileDialog.getSaveFileName(self, 'Save window settings...', self.saveFolder,
                                                   'Bubble settings (*.bub)')[0]
        if fn:
            self.saveFolder = os.path.dirname(fn)
            self.saveSettings(QtCore.QSettings(fn, QtCore.QSettings.IniFormat))

    @QtCore.pyqtSlot(str)
    def on_saxsDetectorComboBox_currentIndexChanged(self, detector):
        self.darkFilesButton.setEnabled(detector == 'Frelon')
        self.applySaxsDarkSignal.emit()

    @QtCore.pyqtSlot(str)
    def on_waxsDetectorComboBox_currentIndexChanged(self, detector):
        self.waxsDarkFilesButton.setEnabled(detector == 'Frelon')
        self.applyWaxsDarkSignal.emit()

    @QtCore.pyqtSlot(name='on_saxsDirClearButton_clicked')
    @QtCore.pyqtSlot(name='on_saxsPoniClearButton_clicked')
    @QtCore.pyqtSlot(name='on_saxsMaskClearButton_clicked')
    @QtCore.pyqtSlot(name='on_saxsBkgClearButton_clicked')
    @QtCore.pyqtSlot(name='on_waxsDirClearButton_clicked')
    @QtCore.pyqtSlot(name='on_waxsPoniClearButton_clicked')
    @QtCore.pyqtSlot(name='on_waxsMaskClearButton_clicked')
    @QtCore.pyqtSlot(name='on_waxsBkgClearButton_clicked')
    def _clearButton_clicked(self):
        clearButtonSlot(self)

    @QtCore.pyqtSlot()
    def on_loadButton_clicked(self):
        fn = QtWidgets.QFileDialog.getOpenFileName(self, 'Load window settings...', self.saveFolder,
                                                   'Bubble settings (*.bub)')[0]
        if fn:
            self.saveFolder = os.path.dirname(fn)
            self.loadSettings(QtCore.QSettings(fn, QtCore.QSettings.IniFormat))

    @QtCore.pyqtSlot()
    def on_waxsMakeMaskButton_clicked(self):
        self.wWaxsMask.show()

    @QtCore.pyqtSlot()
    def on_saxsMakeMaskButton_clicked(self):
        self.wSaxsMask.show()

    @QtCore.pyqtSlot(bool)
    def on_speedCheckBox_toggled(self, checked):
        self.sspeedCheckBox.setEnabled(checked)
        if not checked:
            self.sspeedCheckBox.setChecked(False)
        self.runSignal.emit(self.getSpeed(), 'saxs')
        self.runSignal.emit(self.getSpeed(), 'waxs')

    @QtCore.pyqtSlot(bool)
    def on_sspeedCheckBox_toggled(self, checked):
        self.multiCheckBox.setEnabled(not checked)
        if not self.multiCheckBox.isEnabled():
            self.multiCheckBox.setChecked(False)
        self.runSignal.emit(self.getSpeed(), 'saxs')
        self.runSignal.emit(self.getSpeed(), 'waxs')

    @QtCore.pyqtSlot()
    def on_aboutButton_clicked(self):
        self.wabout.exec()

    @QtCore.pyqtSlot()
    def on_buttonOptions_clicked(self):
        self.wsopts.exec()

    @QtCore.pyqtSlot(bool)
    def on_saxsSlicesCheckBox_toggled(self, checked):
        self.labelSaxsAzimuth.setText('and step' if checked else 'and stop')

    @QtCore.pyqtSlot(bool)
    def on_saxsRadialCheckBox_toggled(self, checked):
        self.saxsRadStartSpinBox.setEnabled(checked)
        self.saxsRadStopSpinBox.setEnabled(checked)

    @QtCore.pyqtSlot(bool)
    def on_saxsAzimuthCheckBox_toggled(self, checked):
        if not checked:
            self.saxsSlicesCheckBox.setChecked(False)
        self.saxsSlicesCheckBox.setEnabled(checked)
        self.saxsAziStartSpinBox.setEnabled(checked)
        self.saxsAziStopSpinBox.setEnabled(checked)

    @QtCore.pyqtSlot(bool)
    def on_waxsSlicesCheckBox_toggled(self, checked):
        self.labelWaxsAzimuth.setText('and step' if checked else 'and stop')

    @QtCore.pyqtSlot(bool)
    def on_waxsRadialCheckBox_toggled(self, checked):
        self.waxsRadStartSpinBox.setEnabled(checked)
        self.waxsRadStopSpinBox.setEnabled(checked)

    @QtCore.pyqtSlot(bool)
    def on_waxsAzimuthCheckBox_toggled(self, checked):
        if not checked:
            self.waxsSlicesCheckBox.setChecked(False)
        self.waxsSlicesCheckBox.setEnabled(checked)
        self.waxsAziStartSpinBox.setEnabled(checked)
        self.waxsAziStopSpinBox.setEnabled(checked)

    @QtCore.pyqtSlot(str)
    def on_comboNorm_currentTextChanged(self, text: str):
        bkg = text == 'Bkg'
        self.spinWaxsBkgStop.setEnabled(bkg)
        self.spinWaxsBkgStart.setEnabled(bkg)

    def checkNewVersion(self):
        if not self.wupdates.cb.isChecked():
            self.createUpdateChecker()
            self.threadUpdateChecker.start()

    def showNewVersion(self, text, error):
        if self.checkUpdatesByUser:
            self.wupdates.error(text) if error else self.wupdates.success(text)
        else:
            if not error and not self.wupdates.cb.isChecked() and text:
                self.wupdates.success(text)

    def checkNewVersionByUser(self):
        self.createUpdateChecker()
        self.checkUpdatesByUser = True
        self.threadUpdateChecker.start()

    def createUpdateChecker(self):
        self.checkUpdatesByUser = False
        self.threadUpdateChecker = QtCore.QThread()
        self.updateChecker = wabout.UpdateChecker()
        self.updateChecker.moveToThread(self.threadUpdateChecker)
        # noinspection PyUnresolvedReferences
        self.threadUpdateChecker.started.connect(self.updateChecker.check)
        self.updateChecker.sigNewVersion.connect(self.showNewVersion)
        self.updateChecker.sigFinished.connect(self.deleteThreadUpdateChecker)

    def deleteThreadUpdateChecker(self):
        self.checkUpdatesByUser = False
        if self.threadUpdateChecker:
            self.threadUpdateChecker.quit()
            self.threadUpdateChecker.wait()
        self.threadUpdateChecker = None
        self.updateChecker = None



#!/usr/bin/python
# -*- coding: utf-8 -*-

import os
from PyQt5 import QtCore, QtWidgets
from .ui.wdark import Ui_WDark
from .fixedwidget import FixedWidget, clearButtonSlot


class WDark(QtWidgets.QDialog, Ui_WDark, FixedWidget):
    def __init__(self, parent=None):
        super().__init__(parent)
        self.setupUi(self)
        self.params = {'darkSample': '', 'darkBackground': '', 'flood': '', 'spline': '', 'incidence': False}
        self.folder = ''
        self.fixWindow()

    def saveSettings(self):
        s = QtCore.QSettings()
        s.setValue('WDark/folder', self.folder)
        s.setValue('WDark/spline', self.splineLineEdit.text())
        s.setValue('WDark/flood', self.floodLineEdit.text())
        s.setValue('WDark/darkbkg', self.bkgLineEdit.text())
        s.setValue('WDark/darksample', self.sampleLineEdit.text())
        s.setValue('WDark/incidence', self.incidenceCheckBox.isChecked())

    def loadSettings(self):
        s = QtCore.QSettings()
        self.folder = s.value('WDark/folder', '', str)
        self.splineLineEdit.setText(s.value('WDark/spline', '', str))
        self.floodLineEdit.setText(s.value('WDark/flood', '', str))
        self.sampleLineEdit.setText(s.value('WDark/darkbkg', '', str))
        self.bkgLineEdit.setText(s.value('WDark/darksample', '', str))
        self.incidenceCheckBox.setChecked(s.value('WDark/incidence', False, bool))
        self.on_buttonBox_accepted()

    @QtCore.pyqtSlot()
    def on_buttonBox_accepted(self):
        self.params = {
            'darkSample': [s.strip() for s in str(self.sampleLineEdit.text()).split(';') if s],
            'darkBackground': [s.strip() for s in str(self.bkgLineEdit.text()).split(';') if s],
            'flood': self.floodLineEdit.text(),
            'spline': self.splineLineEdit.text(),
            'incidence': self.incidenceCheckBox.isChecked(),
        }

    @QtCore.pyqtSlot()
    def on_buttonBox_rejected(self):
        self.bkgLineEdit.setText(';'.join(self.params['darkBackground']))
        self.sampleLineEdit.setText(';'.join(self.params['darkSample']))
        self.floodLineEdit.setText(self.params['flood'])
        self.splineLineEdit.setText(self.params['spline'])
        self.incidenceCheckBox.setChecked(self.params['incidence'])

    @QtCore.pyqtSlot()
    def on_bkgButton_clicked(self):
        self._openButton_clicked(self.bkgLineEdit, self._getFiles)

    @QtCore.pyqtSlot()
    def on_sampleButton_clicked(self):
        self._openButton_clicked(self.sampleLineEdit, self._getFiles)

    @QtCore.pyqtSlot()
    def on_floodButton_clicked(self):
        self._openButton_clicked(self.floodLineEdit, self._getFile)

    @QtCore.pyqtSlot()
    def on_splineButton_clicked(self):
        self._openButton_clicked(self.splineLineEdit, self._getFile, 'Spline file (*.spline)')

    def _getFile(self, lineEdit, current, mask):
        files = QtWidgets.QFileDialog.getOpenFileName(self, 'Select file', current, mask)[0]
        if files:
            lineEdit.setText(files)
            self.folder = os.path.dirname(files)

    def _getFiles(self, lineEdit, current, mask):
        files = QtWidgets.QFileDialog.getOpenFileNames(self, 'Select files', current, mask)[0]
        if files:
            lineEdit.setText(';'.join(files))
            self.folder = os.path.dirname(files[0].split(';')[0])

    def _openButton_clicked(self, lineEdit, func, mask='EDF Images (*.edf)'):
        current = lineEdit.text()
        if current:
            current = os.path.dirname(current.split(';')[0])
        else:
            current = self.folder
        func(lineEdit, current, mask)

    @QtCore.pyqtSlot(name='on_bkgClearButton_clicked')
    @QtCore.pyqtSlot(name='on_sampleClearButton_clicked')
    @QtCore.pyqtSlot(name='on_floodClearButton_clicked')
    @QtCore.pyqtSlot(name='on_splineClearButton_clicked')
    def _clearButton_clicked(self):
        clearButtonSlot(self)



#!/usr/bin/python
# -*- coding: utf-8 -*-

import os
import json
import tempfile
from PyQt5 import QtCore, QtGui, QtWidgets
import numpy as np
import pyqtgraph as pg
import cryio
from .ui.wmask import Ui_WMask


class WMask(QtWidgets.QDialog, Ui_WMask):
    sigMask = QtCore.pyqtSignal(str)

    def __init__(self, itype, parent=None):
        super().__init__(parent)
        self.setupUi(self)
        self.lessEdit.setValidator(QtGui.QIntValidator())
        self.moreEdit.setValidator(QtGui.QIntValidator())
        self.itype = itype
        self.beamline = ''
        self.roi = []
        self.data = None

    def saveSettings(self):
        s = QtCore.QSettings()
        s.setValue('WMask/Geometry', self.saveGeometry())
        s.setValue('WMask/less', self.lessEdit.text())
        s.setValue('WMask/more', self.moreEdit.text())
        s.setValue('WMask/folder', self.folder)

    def loadSettings(self):
        s = QtCore.QSettings()
        self.restoreGeometry(s.value('WMask/Geometry', b''))
        self.lessEdit.setText(s.value('WMask/less', '0', str))
        self.moreEdit.setText(s.value('WMask/more', '0', str))
        self.folder = s.value('WMask/folder', '', str)

    def closeEvent(self, event):
        self.saveSettings()

    def changeBeamline(self, beamline):
        self.beamline = beamline

    @QtCore.pyqtSlot()
    def on_openImageButton_clicked(self):
        image = QtWidgets.QFileDialog.getOpenFileName(self, 'Open image file', self.folder,
                                                      'Frames (*.edf *.cbf *.mar3450)')[0]
        if image:
            self.folder = os.path.dirname(image)
            data = cryio.openImage(image).array
            if self.beamline == 'SNBL':
                data = np.rot90(data[::-1], 3)
            elif self.beamline == 'Dubble':
                if self.itype == 'saxs':
                    data = np.rot90(data, 3)
                elif self.itype == 'waxs':
                    data = np.rot90(data, 1)
            self.data = data
            self.imageView.setImage(data)
            self.setWindowTitle('Mask - {}'.format(image))

    def getImageCenter(self):
        ranges = self.imageView.getView().getState()['viewRange']
        return ranges[0][1] // 2, ranges[1][1] // 2

    @QtCore.pyqtSlot()
    def on_polyButton_clicked(self):
        xc, yc = self.getImageCenter()
        self.addRoi(PolyLineROI([[xc, yc], [xc - 100, yc - 100], [xc - 100, yc + 100]], closed=True, removable=True))

    @QtCore.pyqtSlot()
    def on_ellipseButton_clicked(self):
        xc, yc = self.getImageCenter()
        self.addRoi(EllipseROI([xc, yc], [100, 100], removable=True))

    @QtCore.pyqtSlot()
    def on_buttonCircle_clicked(self):
        xc, yc = self.getImageCenter()
        self.addRoi(CircleROI([xc, yc], [100, 100], removable=True))

    def addRoi(self, roi):
        self.roi.append(roi)
        self.imageView.getView().addItem(roi)
        roi.sigRemoveRequested.connect(self.removeRoi)

    def removeRoi(self, roi):
        self.imageView.getView().removeItem(roi)
        self.roi.remove(roi)

    def makeMask(self):
        image = self.imageView.getImageItem()
        func = np.zeros
        for roi in self.roi:
            if roi.inverted():
                func = np.ones
                break
        mask = func(self.data.shape, dtype=np.int8)
        for roi in self.roi:
            array, coords = roi.getArrayRegion(self.data, image)
            if array is None:
                continue
            array[array != 0] = 1
            coords[coords < 0] = 0
            xcoords, ycoords = coords.round().astype(np.int32)
            xcoords[xcoords > mask.shape[0] - 1] = mask.shape[0] - 1
            ycoords[ycoords > mask.shape[1] - 1] = mask.shape[1] - 1
            if roi.inverted():
                array = 1 - array
            mask[xcoords, ycoords] = array
        less, more = self.getLessMore()
        if less is not None:
            mask[self.data <= less] = 1
        if more is not None:
            mask[self.data >= more] = 1
        return mask

    def getLessMore(self):
        less, more = None, None
        if self.lessCheckbox.isChecked():
            less = int(self.lessEdit.text())
        if self.moreCheckbox.isChecked():
            more = int(self.moreEdit.text())
        return less, more

    def saveApplyMask(self, name, mask):
        np.savez_compressed(name, mask)
        self.sigMask.emit(name)

    @QtCore.pyqtSlot()
    def on_applyButton_clicked(self):
        if self.data is not None:
            mask = self.makeMask()
            name = tempfile.mkstemp(suffix='.npz')[1]
            self.saveApplyMask(name, mask)
            self.close()

    @QtCore.pyqtSlot()
    def on_clearButton_clicked(self):
        for roi in self.roi:
            self.imageView.getView().removeItem(roi)
        self.imageView.clear()
        self.setWindowTitle('Mask')
        self.data = None
        self.roi = []

    @QtCore.pyqtSlot()
    def on_closeButton_clicked(self):
        self.close()

    @QtCore.pyqtSlot()
    def on_saveMaskButton_clicked(self):
        name = QtWidgets.QFileDialog.getSaveFileName(self, 'Save mask', self.folder, 'Bubble mask (*.npz)')[0]
        if name:
            if not name.endswith('.npz'):
                name = '{}.npz'.format(name)
            mask = self.makeMask()
            self.saveApplyMask(name, mask)

    @QtCore.pyqtSlot()
    def on_saveROIButton_clicked(self):
        name = QtWidgets.QFileDialog.getSaveFileName(self, 'Save mask ROI', self.folder, 'Bubble mask ROI (*.roi)')[0]
        if name:
            if not name.endswith('.roi'):
                name = '{}.roi'.format(name)
            less, more = self.getLessMore()
            roi = []
            for r in self.roi:
                state = r.saveState()
                roi.append(state)
            json.dump({'roi': roi, 'less': less, 'more': more}, open(name, 'w'))

    @QtCore.pyqtSlot()
    def on_loadROIButton_clicked(self):
        name = QtWidgets.QFileDialog.getOpenFileName(self, 'Load mask ROI', self.folder, 'Bubble mask ROI (*.roi)')[0]
        if name:
            froi = json.load(open(name, 'r'))
            self.lessEdit.setText(froi['less'])
            self.moreEdit.setText(froi['more'])
            for state in froi['roi']:
                roi = InvertibleROI.getROIByState(state)
                if roi:
                    self.addRoi(roi)
                    roi.setState(state)


class InvertibleROI:
    def __init__(self):
        self.invertedMask = False
        self.actionInvert = None

    def getMenu(self):
        if not self.actionInvert:
            self.actionInvert = QtWidgets.QAction('Invert mask ROI', self.menu)
            self.actionInvert.setCheckable(True)
            self.actionInvert.setChecked(self.invertedMask)
            # noinspection PyUnresolvedReferences
            self.actionInvert.triggered.connect(self.setInverted)
            self.menu.addAction(self.actionInvert)
        return self.menu

    def setInverted(self, checked):
        self.invertedMask = checked

    def setState(self, state):
        if 'invert' in state:
            self.invertedMask = state['invert']
            if self.actionInvert:
                self.actionInvert.setChecked(state['invert'])

    def saveStateI(self, state):
        state['invert'] = self.invertedMask

    def inverted(self):
        return self.invertedMask

    @staticmethod
    def getROIByState(state):
        if 'type' in state:
            if state['type'] == 'PolyLineROI':
                return PolyLineROI([[1, 1], [0, 0], [0, 2]], closed=True, removable=True)
            elif state['type'] == 'EllipseROI':
                return EllipseROI([0, 0], [1, 1], removable=True)
            elif state['type'] == 'CircleROI':
                return CircleROI([0, 0], [1, 1], removable=True)


class PolyLineROI(pg.PolyLineROI, InvertibleROI):
    def __init__(self, positions, closed=False, pos=None, **args):
        super().__init__(positions, closed, pos, **args)
        self.type = 'PolyLineROI'

    def getArrayRegion(self, data, img, axes=(0, 1), **kwds):
        s, m = pg.ROI.getArrayRegion(self, data, img, axes=axes, fromBoundingRect=True, returnMappedCoords=True, **kwds)
        if s is None:
            return None, None
        if img.axisOrder == 'col-major':
            mask = self.renderShapeMask(s.shape[axes[0]], s.shape[axes[1]])
        else:
            mask = self.renderShapeMask(s.shape[axes[1]], s.shape[axes[0]])
            mask = mask.T
        shape = [1] * data.ndim
        shape[axes[0]] = s.shape[axes[0]]
        shape[axes[1]] = s.shape[axes[1]]
        mask = mask.reshape(shape)
        return s * mask, m

    def saveState(self):
        state = super().saveState()
        state['type'] = self.type
        self.saveStateI(state)
        return state

    def getMenu(self):
        super().getMenu()
        return InvertibleROI.getMenu(self)

    def setState(self, state):
        super().setState(state)
        InvertibleROI.setState(self, state)


class EllipseROI(pg.EllipseROI, InvertibleROI):
    def __init__(self, pos, size, **args):
        super().__init__(pos, size, **args)
        self.type = 'EllipseROI'

    def getArrayRegion(self, arr, img=None, axes=(0, 1), **kwds):
        arr, m = pg.ROI.getArrayRegion(self, arr, img, axes, returnMappedCoords=True, **kwds)
        if arr is None or arr.shape[axes[0]] == 0 or arr.shape[axes[1]] == 0:
            return None, None
        w = arr.shape[axes[0]]
        h = arr.shape[axes[1]]
        mask = np.fromfunction(
            lambda x, y: (((x + 0.5) / (w / 2) - 1) ** 2 + ((y + 0.5) / (h / 2) - 1) ** 2) ** 0.5 < 1, (w, h))
        if axes[0] > axes[1]:
            mask = mask.T
        shape = [(n if i in axes else 1) for i, n in enumerate(arr.shape)]
        mask = mask.reshape(shape)
        return arr * mask, m

    def saveState(self):
        state = super().saveState()
        state['type'] = self.type
        self.saveStateI(state)
        return state

    def getMenu(self):
        super().getMenu()
        return InvertibleROI.getMenu(self)

    def setState(self, state, update=True):
        super().setState(state, update)
        InvertibleROI.setState(self, state)


class CircleROI(EllipseROI):
    def __init__(self, pos, size, **args):
        super().__init__(pos, size, **args)
        self.aspectLocked = True
        self.addScaleHandle([0.5 * 2. ** -0.5 + 0.5, 0.5 * 2. ** -0.5 + 0.5], [0.5, 0.5])
        self.type = 'CircleROI'



#!/usr/bin/python
# -*- coding: utf-8 -*-

import sys
from PyQt5 import QtWidgets


app = None


def main():
    global app
    app = QtWidgets.QApplication(sys.argv)
    app.setOrganizationName('Dubble')
    app.setOrganizationDomain('esrf.eu')
    app.setApplicationName('bubble')
    from .wbubblec import WBubble
    wbubble = WBubble()
    wbubble.start()
    sys.exit(app.exec())



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'bubble/bclient/ui/wsaxsparams.ui'
#
# Created by: PyQt5 UI code generator 5.8.2
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_WSaxsParams(object):
    def setupUi(self, WSaxsParams):
        WSaxsParams.setObjectName("WSaxsParams")
        WSaxsParams.resize(578, 236)
        self.formLayout = QtWidgets.QFormLayout(WSaxsParams)
        self.formLayout.setObjectName("formLayout")
        self.label = QtWidgets.QLabel(WSaxsParams)
        self.label.setLocale(QtCore.QLocale(QtCore.QLocale.English, QtCore.QLocale.UnitedStates))
        self.label.setObjectName("label")
        self.formLayout.setWidget(0, QtWidgets.QFormLayout.LabelRole, self.label)
        self.thicknessLineEdit = QtWidgets.QLineEdit(WSaxsParams)
        self.thicknessLineEdit.setMinimumSize(QtCore.QSize(200, 0))
        self.thicknessLineEdit.setLocale(QtCore.QLocale(QtCore.QLocale.English, QtCore.QLocale.UnitedStates))
        self.thicknessLineEdit.setText("")
        self.thicknessLineEdit.setObjectName("thicknessLineEdit")
        self.formLayout.setWidget(0, QtWidgets.QFormLayout.FieldRole, self.thicknessLineEdit)
        self.label_2 = QtWidgets.QLabel(WSaxsParams)
        self.label_2.setLocale(QtCore.QLocale(QtCore.QLocale.English, QtCore.QLocale.UnitedStates))
        self.label_2.setObjectName("label_2")
        self.formLayout.setWidget(1, QtWidgets.QFormLayout.LabelRole, self.label_2)
        self.concentrationLineEdit = QtWidgets.QLineEdit(WSaxsParams)
        self.concentrationLineEdit.setMinimumSize(QtCore.QSize(200, 0))
        self.concentrationLineEdit.setLocale(QtCore.QLocale(QtCore.QLocale.English, QtCore.QLocale.UnitedStates))
        self.concentrationLineEdit.setText("")
        self.concentrationLineEdit.setObjectName("concentrationLineEdit")
        self.formLayout.setWidget(1, QtWidgets.QFormLayout.FieldRole, self.concentrationLineEdit)
        self.label_3 = QtWidgets.QLabel(WSaxsParams)
        self.label_3.setLocale(QtCore.QLocale(QtCore.QLocale.English, QtCore.QLocale.UnitedStates))
        self.label_3.setObjectName("label_3")
        self.formLayout.setWidget(2, QtWidgets.QFormLayout.LabelRole, self.label_3)
        self.calibrationLineEdit = QtWidgets.QLineEdit(WSaxsParams)
        self.calibrationLineEdit.setMinimumSize(QtCore.QSize(200, 0))
        self.calibrationLineEdit.setLocale(QtCore.QLocale(QtCore.QLocale.English, QtCore.QLocale.UnitedStates))
        self.calibrationLineEdit.setText("")
        self.calibrationLineEdit.setObjectName("calibrationLineEdit")
        self.formLayout.setWidget(2, QtWidgets.QFormLayout.FieldRole, self.calibrationLineEdit)
        self.buttonBox = QtWidgets.QDialogButtonBox(WSaxsParams)
        self.buttonBox.setLocale(QtCore.QLocale(QtCore.QLocale.English, QtCore.QLocale.UnitedStates))
        self.buttonBox.setOrientation(QtCore.Qt.Horizontal)
        self.buttonBox.setStandardButtons(QtWidgets.QDialogButtonBox.Cancel|QtWidgets.QDialogButtonBox.Ok)
        self.buttonBox.setObjectName("buttonBox")
        self.formLayout.setWidget(3, QtWidgets.QFormLayout.SpanningRole, self.buttonBox)

        self.retranslateUi(WSaxsParams)
        self.buttonBox.accepted.connect(WSaxsParams.accept)
        self.buttonBox.rejected.connect(WSaxsParams.reject)
        QtCore.QMetaObject.connectSlotsByName(WSaxsParams)

    def retranslateUi(self, WSaxsParams):
        _translate = QtCore.QCoreApplication.translate
        WSaxsParams.setWindowTitle(_translate("WSaxsParams", "Advanced SAXS parameters"))
        self.label.setText(_translate("WSaxsParams", "Sample thickness (cm)"))
        self.label_2.setText(_translate("WSaxsParams", "Sample concentration (g/ml)"))
        self.label_3.setText(_translate("WSaxsParams", "<html><head/><body><p>Calibration factor (cm<span style=\" vertical-align:super;\">-1</span>)</p></body></html>"))




#!/usr/bin/python
# -*- coding: utf-8 -*-

import io
import zlib
import base64
import binascii
import numpy as np


class CompressError(Exception):
    pass


def compress(s, text=True):
    s = zlib.compress(s.encode(), 9)
    if text:
        s = base64.b64encode(s).decode()
    return s


def compressFile(f, text=True):
    try:
        return compress(open(f).read(), text)
    except IOError:
        return ''


def decompress(bts, text=True):
    try:
        if text:
            bts = base64.b64decode(bts)
        return zlib.decompress(bts)
    except (binascii.Error, zlib.error):
        raise CompressError('The string does not seem to be compressed')


def compressNumpyArray(array, text=True):
    buf = io.BytesIO()
    # noinspection PyTypeChecker
    np.savez_compressed(buf, array)
    bts = buf.getvalue()
    if text:
        bts = base64.b64encode(bts).decode()
    return bts


def decompressNumpyArray(array, text=True):
    try:
        if text:
            array = base64.b64decode(array)
        return np.load(io.BytesIO(array))['arr_0']
    except (TypeError, OSError, binascii.Error):
        return None



#!/usr/bin/python
# -*- coding: utf-8 -*-

import json
import asyncio
from .worker import BubbleWorker


class BubbleProtocol(asyncio.Protocol):
    worker = BubbleWorker()

    def connection_made(self, transport):
        self.transport = transport
        self.buffer = b''
        peername = transport.get_extra_info('peername')
        print('Connection from {}:{:d}'.format(*peername))
        self.worker.resetSent()

    def data_received(self, data):
        self.buffer += data
        buf = self.buffer.split(b'\r\n')
        self.buffer = buf[-1]
        for request in buf[:-1]:
            try:
                decoded_request = json.loads(request.decode())
            except json.JSONDecodeError:
                return
            response = self.worker.parse(decoded_request)
            self.transport.write(response)



#!/usr/bin/python
# -*- coding: utf-8 -*-

import os
import time
import glob
import asyncio
from concurrent import futures
import numpy as np
from ..bcommon import compressor, default
from . import interface
from .integrator import Integrator

ATTEMPTS_TO_OPEN = 10


class PeriodicTask:
    def __init__(self, func):
        self.func = func
        self.stopped = True
        self.running = False

    def start(self, interval):
        self.interval = interval
        self.running = True
        asyncio.ensure_future(self._run())

    async def _run(self):
        self.stopped = False
        while self.running:
            self.func()
            await asyncio.sleep(self.interval)
        self.stopped = True

    def stop(self):
        self.running = False

    async def stopping(self):
        while not self.stopped:
            await asyncio.sleep(self.interval)


class Watcher:
    def __init__(self, typ='waxs'):
        self.interface = interface.Isaxs() if typ == 'saxs' else interface.Iwaxs()
        self._path = ''
        self._threads = 0
        self.runLocally = False
        self._multicolumnName = ''
        self.all = 0
        self.watchdog = PeriodicTask(self.checkFiles)
        self._loop = asyncio.get_event_loop()
        self.__integrator = Integrator()
        self.setPool()
        self.clearState()

    def clearState(self):
        self.results = {
            'imageFile': '',
            'chiFile': '',
            'transmission': 0,
            'data1d': None,
            'image': None,
            'timestamp': time.time(),
            'sent': True,
        }
        self.errors = []
        self.warnings = []
        self.taken = set()
        self.multicolumn = []
        self.qvalues = None
        self.all = 0
        self.interface.clearState()
        self.generate_response()

    @property
    def path(self):
        return self._path if self._path else 'unknown'

    @path.setter
    def path(self, path):
        if self._path == path:
            return
        elif not os.path.exists(path):
            self.errors.append('The {} path "{}" does not exist'.format(self.interface.type, path))
        elif not os.path.isdir(path):
            self.errors.append('The {} path "{}" is not a folder'.format(self.interface.type, path))
        else:
            self._changePath(path)

    def _changePath(self, path):
        run = self.watchdog.running
        if run:
            self.stopWatching()
        self._path = path
        if run:
            self.runWatching()

    def runWatching(self):
        asyncio.ensure_future(self._runWatching())

    async def _runWatching(self):
        if not self.watchdog.running:
            self.clearState()
            await self._loop.run_in_executor(self._pool, self.interface.init)
            self.watchdog.start(default.WATCHDOG_CALL_TIMEOUT)

    def checkFiles(self):
        edfs = glob.glob(os.path.join(self._path, '*.edf'))
        cbfs = glob.glob(os.path.join(self._path, '*.cbf'))
        mar3450 = glob.glob(os.path.join(self._path, '*.mar3450'))
        mar2300 = glob.glob(os.path.join(self._path, '*.mar2300'))
        images = set(edfs + cbfs + mar3450 + mar2300)
        self.all = len(images)
        images -= self.taken
        if self.interface.speed and self.interface.sspeed:
            self.taken |= images
            for image in images:
                asyncio.ensure_future(self._integrate_in_thread(image))
        else:
            for image in images:
                if not self.watchdog.running:
                    return
                self.taken.add(image)
                asyncio.ensure_future(self.integrate(image))

    def _integrate_in_thread(self, image):
        return self._loop.run_in_executor(self._pool, self.__integrator, image, self.interface)

    async def integrate(self, image):
        results = await self._integrate_in_thread(image)
        if results:
            self.results = results
            self.resetSent()
            if self.multicolumnName:
                self.multicolumn.append((results['chiFile'], results['data1d'][1]))
                if self.qvalues is None:
                    self.qvalues = results['data1d'][0]

    def stopWatching(self):
        if self.watchdog.running:
            self.watchdog.stop()
            self.shutdown_tasks()
            if self.multicolumn:
                asyncio.ensure_future(self._loop.run_in_executor(None, self.save_multicolumn))

    def shutdown_tasks(self):
        for task in asyncio.Task.all_tasks():
            task.cancel()

    def save_multicolumn(self):
        self.multicolumn.sort()
        arrays, header = [compressor.decompressNumpyArray(self.qvalues, not self.interface.pickle)], 'q '
        for fn, ar in self.multicolumn:
            arrays.append(compressor.decompressNumpyArray(ar, not self.interface.pickle))
            header += '{} '.format(os.path.basename(fn))
        outf = os.path.join(self._path, self.interface.subdir, default.MULTICOLUMN_NAME)
        np.savetxt(outf, np.array(arrays).transpose(), fmt='%.6e', header=header, comments='#')

    @property
    def multicolumnName(self):
        return self._multicolumnName

    @multicolumnName.setter
    def multicolumnName(self, name):
        self._multicolumnName = name or ''

    def setParameters(self, params):
        self.interface.errors = self.errors = []
        self.interface.warnings = self.warnings = []
        for param in params:
            setattr(self, param, params[param])
            setattr(self.interface, param, params[param])
        if 'stop' in params:
            self.stopWatching()
        if not self.errors and params.get('run', 0):
            self.runWatching()

    def state(self):
        return self.generate_response()

    def generate_response(self):
        self.response = {
            'running': self.watchdog.running,
            'chiFile': self.results['chiFile'],
            'path': self.path,
            'total': self.total(),
            'imageFile': self.results['imageFile'],
            'transmission': self.results['transmission'],
            'data1d': None,
            'image': None,
            'timestamp': self.results['timestamp'],
            'all': self.all,
        }
        if not self.results['sent']:
            if self.results['data1d'] is not None:
                self.response['data1d'] = self.results['data1d']
            if self.results['image'] is not None:
                self.response['image'] = self.results['image']
            self.results['sent'] = True
        return self.response

    @property
    def threads(self):
        return self._threads

    @threads.setter
    def threads(self, n):
        if self._threads != n:
            self._threads = n
            self.setPool()

    def setPool(self):
        if self._threads <= 0:
            self._threads = os.cpu_count()
            if not self._threads:
                self._threads = default.DEFAULT_THREADS
        if self._threads > default.MAX_THREADS:
            print(f'To many threads ({self._threads}) are not supported, setting number of threads to '
                  f'{default.MAX_THREADS}')
            self._threads = default.MAX_THREADS
        print(f'Running integration in {self._threads} threads')
        self._pool = futures.ThreadPoolExecutor(self._threads)

    def resetSent(self):
        self.results['sent'] = False

    def total(self):
        return self.interface.counter



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'bubble/bclient/ui/wmask.ui'
#
# Created by: PyQt5 UI code generator 5.8.2
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_WMask(object):
    def setupUi(self, WMask):
        WMask.setObjectName("WMask")
        WMask.resize(883, 638)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/mask"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        WMask.setWindowIcon(icon)
        self.verticalLayout = QtWidgets.QVBoxLayout(WMask)
        self.verticalLayout.setObjectName("verticalLayout")
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.checkInvert = QtWidgets.QCheckBox(WMask)
        self.checkInvert.setObjectName("checkInvert")
        self.horizontalLayout.addWidget(self.checkInvert)
        self.lessCheckbox = QtWidgets.QCheckBox(WMask)
        self.lessCheckbox.setObjectName("lessCheckbox")
        self.horizontalLayout.addWidget(self.lessCheckbox)
        self.lessEdit = QtWidgets.QLineEdit(WMask)
        self.lessEdit.setObjectName("lessEdit")
        self.horizontalLayout.addWidget(self.lessEdit)
        self.horizontalLayout_3.addLayout(self.horizontalLayout)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.moreCheckbox = QtWidgets.QCheckBox(WMask)
        self.moreCheckbox.setObjectName("moreCheckbox")
        self.horizontalLayout_2.addWidget(self.moreCheckbox)
        self.moreEdit = QtWidgets.QLineEdit(WMask)
        self.moreEdit.setObjectName("moreEdit")
        self.horizontalLayout_2.addWidget(self.moreEdit)
        self.horizontalLayout_3.addLayout(self.horizontalLayout_2)
        self.polyButton = QtWidgets.QPushButton(WMask)
        self.polyButton.setObjectName("polyButton")
        self.horizontalLayout_3.addWidget(self.polyButton)
        self.ellipseButton = QtWidgets.QPushButton(WMask)
        self.ellipseButton.setObjectName("ellipseButton")
        self.horizontalLayout_3.addWidget(self.ellipseButton)
        self.buttonCircle = QtWidgets.QPushButton(WMask)
        self.buttonCircle.setObjectName("buttonCircle")
        self.horizontalLayout_3.addWidget(self.buttonCircle)
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_3.addItem(spacerItem)
        self.verticalLayout.addLayout(self.horizontalLayout_3)
        self.imageView = ImageView(WMask)
        self.imageView.setObjectName("imageView")
        self.verticalLayout.addWidget(self.imageView)
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        self.openImageButton = QtWidgets.QPushButton(WMask)
        self.openImageButton.setObjectName("openImageButton")
        self.horizontalLayout_4.addWidget(self.openImageButton)
        self.applyButton = QtWidgets.QPushButton(WMask)
        self.applyButton.setObjectName("applyButton")
        self.horizontalLayout_4.addWidget(self.applyButton)
        self.saveMaskButton = QtWidgets.QPushButton(WMask)
        self.saveMaskButton.setObjectName("saveMaskButton")
        self.horizontalLayout_4.addWidget(self.saveMaskButton)
        self.saveROIButton = QtWidgets.QPushButton(WMask)
        self.saveROIButton.setObjectName("saveROIButton")
        self.horizontalLayout_4.addWidget(self.saveROIButton)
        self.loadROIButton = QtWidgets.QPushButton(WMask)
        self.loadROIButton.setObjectName("loadROIButton")
        self.horizontalLayout_4.addWidget(self.loadROIButton)
        spacerItem1 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_4.addItem(spacerItem1)
        self.clearButton = QtWidgets.QPushButton(WMask)
        self.clearButton.setObjectName("clearButton")
        self.horizontalLayout_4.addWidget(self.clearButton)
        self.closeButton = QtWidgets.QPushButton(WMask)
        self.closeButton.setObjectName("closeButton")
        self.horizontalLayout_4.addWidget(self.closeButton)
        self.verticalLayout.addLayout(self.horizontalLayout_4)

        self.retranslateUi(WMask)
        QtCore.QMetaObject.connectSlotsByName(WMask)

    def retranslateUi(self, WMask):
        _translate = QtCore.QCoreApplication.translate
        WMask.setWindowTitle(_translate("WMask", "Mask"))
        self.checkInvert.setText(_translate("WMask", "Invert ROI"))
        self.lessCheckbox.setText(_translate("WMask", "Less than"))
        self.moreCheckbox.setText(_translate("WMask", "More than"))
        self.polyButton.setText(_translate("WMask", "Polygone"))
        self.ellipseButton.setText(_translate("WMask", "Ellipse"))
        self.buttonCircle.setText(_translate("WMask", "Circle"))
        self.openImageButton.setText(_translate("WMask", "Open image"))
        self.applyButton.setText(_translate("WMask", "Apply mask"))
        self.saveMaskButton.setText(_translate("WMask", "Save mask"))
        self.saveROIButton.setText(_translate("WMask", "Save ROIs"))
        self.loadROIButton.setText(_translate("WMask", "Load ROIs"))
        self.clearButton.setText(_translate("WMask", "Clear"))
        self.closeButton.setText(_translate("WMask", "Close"))

from pyqtgraph import ImageView
from . import resources_rc



#!/usr/bin/python
# -*- coding: utf-8 -*-

import os
import threading
import decor
from integracio import igracio
from ..bcommon import compressor


class _Interface:
    type = 'None'
    calcTransmission = False

    def __init__(self):
        self.errors = []
        self.warnings = []
        self.i = igracio.IGracio()
        self._precision = self.i.precision
        self._lock = threading.RLock()
        self._counter = 0
        self._maskFile = None
        self._pickle = False
        self._floodFile = None
        self._splineFile = None
        self._backgroundFiles = []
        self._darkSampleFiles = []
        self._darkBackgroundFiles = []
        self._beamline = 'Dubble'
        self._concentration = 0
        self._calibration = 1
        self._units = 'q'
        self._ext = ''
        self._thickness = 1
        self._subdir = ''
        self._detector = 'Pilatus'
        self._save = False
        self._azimuth = None
        self._radial = None
        self._bkgCoef = 0
        self._speed = True
        self._sspeed = False
        self._poni = ''
        self._azimuth_checked = False
        self._azimuth_slices = False
        self._incidence = False
        self._normalization = 'Monitor'
        self._normrange = 0, 0
        self._threads = 0
        self._polarization = -2
        self._halina = False
        self._solidangle = True
        self._bins = 0
        self.wavelength = 1
        self.normalize = decor.Normalization()
        self.mask = decor.Mask()
        self.background = decor.Background()
        self.darksample = decor.DarkCurrent()
        self.darkbackground = decor.DarkCurrent()
        self.flatfield = decor.FloodField()
        self.distortion = decor.Distortion()

    def init(self):
        self.i.poni = self.poni
        self.i.units = self.units
        self.i.radial = self.radial
        self.i.polarization = self.polarization
        self.i.sa = self.solidangle
        self.i.bins = self.bins
        self.normalize.beamline = self.beamline
        self.normalize.type = self.normalization
        self.normalize.range = self.normrange
        self.wavelength = self.i.poni.wavelength / 10  # nm
        if self.detector == 'Frelon':
            ds = self.darkSample
            db = self.darkBackground
            flood = self.flood
            spline = self.spline
        else:
            ds = db = flood = spline = None
        self.darksample.init(ds)
        self.darkbackground.init(db)
        self.flatfield.init(flood)
        self.distortion.init(spline)
        self.mask.init(self.maskFile, self.beamline, self.type)
        self.background.init(self.backgroundFiles, self.darkbackground, self.normalize, self.bkgCoef, self.flatfield)

    @property
    def backgroundFiles(self):
        return self._backgroundFiles

    @backgroundFiles.setter
    def backgroundFiles(self, bkg):
        self._backgroundFiles = []
        if bkg:
            self._backgroundFiles = [b for b in bkg if os.path.exists(b)]
            if not self._backgroundFiles:
                self.warnings.append('Not all the {} background files could be found'.format(self.type))

    @property
    def darkBackground(self):
        return self._darkBackgroundFiles

    @darkBackground.setter
    def darkBackground(self, dark):
        self._darkBackgroundFiles = []
        if dark:
            self._darkBackgroundFiles = [d for d in dark if os.path.exists(d)]
            if not self._darkBackgroundFiles:
                self.warnings.append('Not all the {} dark background files could be found'.format(self.type))

    @property
    def darkSample(self):
        return self._darkSampleFiles

    @darkSample.setter
    def darkSample(self, dark):
        self._darkSampleFiles = []
        if dark:
            self._darkSampleFiles = [d for d in dark if os.path.exists(d)]
            if not self._darkSampleFiles:
                self.warnings.append('Not all the {} dark sample files could be found'.format(self.type))

    @property
    def poni(self):
        return self._poni

    @poni.setter
    def poni(self, filename):
        if not filename:
            self.errors.append('The {} poni file must be specified'.format(self.type))
            return
        if os.path.exists(filename):
            self._poni = open(filename).read()
        else:
            try:
                self._poni = compressor.decompress(filename).decode()
            except compressor.CompressError:
                self.errors.append('The {} poni string seems to be corrupted'.format(self.type))

    @property
    def maskFile(self):
        return self._maskFile

    @maskFile.setter
    def maskFile(self, mask):
        if not mask:
            self._maskFile = None
        else:
            _mask = compressor.decompressNumpyArray(mask)
            if _mask is not None:
                self._maskFile = _mask
            else:
                if os.path.exists(mask):
                    self._maskFile = mask
                else:
                    self._maskFile = None
                    self.warnings.append('The {} mask file "{}" cannot be found. '
                                         'Run without mask'.format(self.type, mask))

    @property
    def spline(self):
        return self._splineFile

    @spline.setter
    def spline(self, spline):
        self._splineFile = None
        if spline:
            if os.path.exists(spline):
                self._splineFile = spline
            else:
                self.warnings.append('The {} spline file "{}" cannot be found'.format(self.type, spline))

    @property
    def flood(self):
        return self._floodFile

    @flood.setter
    def flood(self, flood):
        self._floodFile = None
        if flood:
            if os.path.exists(flood):
                self._floodFile = flood
            else:
                self.warnings.append('The {} flood file "{}" cannot be found'.format(self.type, flood))

    @property
    def bkgCoef(self):
        return self._bkgCoef

    @bkgCoef.setter
    def bkgCoef(self, coef):
        self._bkgCoef = coef

    @property
    def radial(self):
        return self._radial

    @radial.setter
    def radial(self, values):
        self._radial = values

    @property
    def azimuth(self):
        return self._azimuth

    @azimuth.setter
    def azimuth(self, values):
        self._azimuth = values

    @property
    def azimuthChecked(self):
        return self._azimuth_checked

    @azimuthChecked.setter
    def azimuthChecked(self, value):
        self._azimuth_checked = value

    @property
    def azimuthSlices(self):
        return self._azimuth_slices

    @azimuthSlices.setter
    def azimuthSlices(self, value):
        self._azimuth_slices = value

    @property
    def beamline(self):
        return self._beamline

    @beamline.setter
    def beamline(self, beamline):
        self._beamline = beamline

    @property
    def thickness(self):
        return self._thickness

    @thickness.setter
    def thickness(self, thickness):
        self._thickness = thickness

    @property
    def concentration(self):
        return self._concentration

    @concentration.setter
    def concentration(self, concentration):
        self._concentration = concentration

    @property
    def calibration(self):
        return self._calibration

    @calibration.setter
    def calibration(self, calibration):
        self._calibration = calibration

    @property
    def units(self):
        return self._units

    @units.setter
    def units(self, units):
        self._units = units

    @property
    def ext(self):
        return self._ext

    @ext.setter
    def ext(self, ext):
        self._ext = ext

    @property
    def subdir(self):
        return self._subdir

    @subdir.setter
    def subdir(self, subdir):
        self._subdir = subdir

    @property
    def detector(self):
        return self._detector

    @detector.setter
    def detector(self, detector):
        self._detector = detector

    @property
    def save(self):
        return self._save

    @save.setter
    def save(self, save):
        self._save = save

    @property
    def speed(self):
        with self._lock:
            return self._speed

    @speed.setter
    def speed(self, speed):
        with self._lock:
            self._speed = speed

    @property
    def sspeed(self):
        with self._lock:
            return self._sspeed

    @sspeed.setter
    def sspeed(self, sspeed):
        with self._lock:
            self._sspeed = sspeed

    def clearState(self):
        self.counter = 0

    @property
    def counter(self):
        with self._lock:
            return self._counter

    @counter.setter
    def counter(self, value):
        with self._lock:
            self._counter = value

    @property
    def pickle(self):
        return self._pickle

    @pickle.setter
    def pickle(self, v):
        self._pickle = v

    @property
    def incidence(self):
        return self._incidence

    @incidence.setter
    def incidence(self, v):
        self._incidence = v

    @property
    def normalization(self):
        return self._normalization

    @normalization.setter
    def normalization(self, v):
        self._normalization = v

    @property
    def normrange(self):
        return self._normrange

    @normrange.setter
    def normrange(self, v):
        if isinstance(v, (tuple, list)) and len(v) == 2:
            self._normrange = v
        else:
            self._normrange = 0, 0

    @property
    def is_monitor(self):
        return self._normalization == 'Monitor'

    @property
    def double(self):
        return self._precision == 'double'

    @double.setter
    def double(self, value: bool):
        self._precision = 'double' if value else 'float'
        self.i = igracio.IGracio(precision=self._precision)

    @property
    def polarization(self):
        return self._polarization

    @polarization.setter
    def polarization(self, value: float):
        self._polarization = value

    @property
    def halina(self):
        return self._halina

    @halina.setter
    def halina(self, value: bool):
        self._halina = value

    @property
    def solidangle(self):
        return self._solidangle

    @solidangle.setter
    def solidangle(self, value: bool):
        self._solidangle = value

    @property
    def bins(self):
        return self._bins

    @bins.setter
    def bins(self, value: bool):
        self._bins = value

    @property
    def threads(self):
        return self._threads

    @threads.setter
    def threads(self, value: int):
        self._threads = value


class Iwaxs(_Interface):
    def __init__(self):
        super().__init__()
        self.type = 'waxs'


class Isaxs(_Interface):
    def __init__(self):
        super().__init__()
        self.type = 'saxs'
        self.calcTransmission = True



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import sys
import subprocess
try:
    from bubble import frozen
except ImportError:
    frozen = False


def get_hg_hash():
    if not get_hg_hash.hash:
        # noinspection PyUnresolvedReferences
        if hasattr(sys, 'frozen') or frozen:
            get_hg_hash.hash = frozen.hg_hash
        else:
            path = os.path.dirname(os.path.dirname(os.path.dirname(__file__)))
            try:
                pipe = subprocess.Popen(['hg', 'id', '-i', '-R', path], stdout=subprocess.PIPE)
                get_hg_hash.hash = pipe.stdout.read().decode()
            except OSError:
                get_hg_hash.hash = 'unknown'
        get_hg_hash.hash = get_hg_hash.hash.strip()
    return get_hg_hash.hash


get_hg_hash.hash = ''


def get_version():
    if not get_version.version:
        if hasattr(sys, 'frozen') or frozen:
            get_version.version = frozen.version
        else:
            get_version.version = '0.0.0'
    return get_version.version


get_version.version = ''



#!/usr/bin/python
# -*- coding: utf-8 -*-

import struct


DEFAULT_PORT = 8587
DEFAULT_HOST = 'localhost'
WATCHDOG_STARTSTOP_TIMEOUT = 0
WATCHDOG_CALL_TIMEOUT = 0.3
MULTICOLUMN_NAME = 'allfiles.out'
PICKLE_MAGIC_NUMBER = 0xdeadbeef
PICKLE_HEADER_STRUCT = 'QQ'
PICKLE_HEADER_SIZE = struct.calcsize(PICKLE_HEADER_STRUCT)
MAX_THREADS = 50
DEFAULT_THREADS = 3



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'bubble/bclient/ui/wdark.ui'
#
# Created by: PyQt5 UI code generator 5.8.2
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_WDark(object):
    def setupUi(self, WDark):
        WDark.setObjectName("WDark")
        WDark.resize(759, 348)
        WDark.setLocale(QtCore.QLocale(QtCore.QLocale.English, QtCore.QLocale.UnitedStates))
        self.verticalLayout = QtWidgets.QVBoxLayout(WDark)
        self.verticalLayout.setObjectName("verticalLayout")
        self.gridLayout = QtWidgets.QGridLayout()
        self.gridLayout.setObjectName("gridLayout")
        self.label = QtWidgets.QLabel(WDark)
        self.label.setObjectName("label")
        self.gridLayout.addWidget(self.label, 0, 0, 1, 1)
        self.bkgLineEdit = QtWidgets.QLineEdit(WDark)
        self.bkgLineEdit.setMinimumSize(QtCore.QSize(400, 0))
        self.bkgLineEdit.setLocale(QtCore.QLocale(QtCore.QLocale.English, QtCore.QLocale.UnitedStates))
        self.bkgLineEdit.setText("")
        self.bkgLineEdit.setObjectName("bkgLineEdit")
        self.gridLayout.addWidget(self.bkgLineEdit, 0, 1, 1, 1)
        self.bkgButton = QtWidgets.QPushButton(WDark)
        self.bkgButton.setText("")
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/folder"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.bkgButton.setIcon(icon)
        self.bkgButton.setObjectName("bkgButton")
        self.gridLayout.addWidget(self.bkgButton, 0, 2, 1, 1)
        self.bkgClearButton = QtWidgets.QPushButton(WDark)
        self.bkgClearButton.setText("")
        icon1 = QtGui.QIcon()
        icon1.addPixmap(QtGui.QPixmap(":/clear"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.bkgClearButton.setIcon(icon1)
        self.bkgClearButton.setObjectName("bkgClearButton")
        self.gridLayout.addWidget(self.bkgClearButton, 0, 3, 1, 1)
        self.label_2 = QtWidgets.QLabel(WDark)
        self.label_2.setObjectName("label_2")
        self.gridLayout.addWidget(self.label_2, 1, 0, 1, 1)
        self.sampleLineEdit = QtWidgets.QLineEdit(WDark)
        self.sampleLineEdit.setMinimumSize(QtCore.QSize(400, 0))
        self.sampleLineEdit.setLocale(QtCore.QLocale(QtCore.QLocale.English, QtCore.QLocale.UnitedStates))
        self.sampleLineEdit.setText("")
        self.sampleLineEdit.setObjectName("sampleLineEdit")
        self.gridLayout.addWidget(self.sampleLineEdit, 1, 1, 1, 1)
        self.sampleButton = QtWidgets.QPushButton(WDark)
        self.sampleButton.setText("")
        self.sampleButton.setIcon(icon)
        self.sampleButton.setObjectName("sampleButton")
        self.gridLayout.addWidget(self.sampleButton, 1, 2, 1, 1)
        self.sampleClearButton = QtWidgets.QPushButton(WDark)
        self.sampleClearButton.setText("")
        self.sampleClearButton.setIcon(icon1)
        self.sampleClearButton.setObjectName("sampleClearButton")
        self.gridLayout.addWidget(self.sampleClearButton, 1, 3, 1, 1)
        self.label_3 = QtWidgets.QLabel(WDark)
        self.label_3.setObjectName("label_3")
        self.gridLayout.addWidget(self.label_3, 2, 0, 1, 1)
        self.splineLineEdit = QtWidgets.QLineEdit(WDark)
        self.splineLineEdit.setMinimumSize(QtCore.QSize(400, 0))
        self.splineLineEdit.setLocale(QtCore.QLocale(QtCore.QLocale.English, QtCore.QLocale.UnitedStates))
        self.splineLineEdit.setText("")
        self.splineLineEdit.setObjectName("splineLineEdit")
        self.gridLayout.addWidget(self.splineLineEdit, 2, 1, 1, 1)
        self.splineButton = QtWidgets.QPushButton(WDark)
        self.splineButton.setText("")
        self.splineButton.setIcon(icon)
        self.splineButton.setObjectName("splineButton")
        self.gridLayout.addWidget(self.splineButton, 2, 2, 1, 1)
        self.splineClearButton = QtWidgets.QPushButton(WDark)
        self.splineClearButton.setText("")
        self.splineClearButton.setIcon(icon1)
        self.splineClearButton.setObjectName("splineClearButton")
        self.gridLayout.addWidget(self.splineClearButton, 2, 3, 1, 1)
        self.label_4 = QtWidgets.QLabel(WDark)
        self.label_4.setObjectName("label_4")
        self.gridLayout.addWidget(self.label_4, 3, 0, 1, 1)
        self.floodLineEdit = QtWidgets.QLineEdit(WDark)
        self.floodLineEdit.setMinimumSize(QtCore.QSize(400, 0))
        self.floodLineEdit.setLocale(QtCore.QLocale(QtCore.QLocale.English, QtCore.QLocale.UnitedStates))
        self.floodLineEdit.setText("")
        self.floodLineEdit.setObjectName("floodLineEdit")
        self.gridLayout.addWidget(self.floodLineEdit, 3, 1, 1, 1)
        self.floodButton = QtWidgets.QPushButton(WDark)
        self.floodButton.setText("")
        self.floodButton.setIcon(icon)
        self.floodButton.setObjectName("floodButton")
        self.gridLayout.addWidget(self.floodButton, 3, 2, 1, 1)
        self.floodClearButton = QtWidgets.QPushButton(WDark)
        self.floodClearButton.setText("")
        self.floodClearButton.setIcon(icon1)
        self.floodClearButton.setObjectName("floodClearButton")
        self.gridLayout.addWidget(self.floodClearButton, 3, 3, 1, 1)
        self.verticalLayout.addLayout(self.gridLayout)
        self.incidenceCheckBox = QtWidgets.QCheckBox(WDark)
        self.incidenceCheckBox.setObjectName("incidenceCheckBox")
        self.verticalLayout.addWidget(self.incidenceCheckBox)
        spacerItem = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout.addItem(spacerItem)
        self.buttonBox = QtWidgets.QDialogButtonBox(WDark)
        self.buttonBox.setOrientation(QtCore.Qt.Horizontal)
        self.buttonBox.setStandardButtons(QtWidgets.QDialogButtonBox.Cancel|QtWidgets.QDialogButtonBox.Ok)
        self.buttonBox.setObjectName("buttonBox")
        self.verticalLayout.addWidget(self.buttonBox)

        self.retranslateUi(WDark)
        self.buttonBox.accepted.connect(WDark.accept)
        self.buttonBox.rejected.connect(WDark.reject)
        QtCore.QMetaObject.connectSlotsByName(WDark)

    def retranslateUi(self, WDark):
        _translate = QtCore.QCoreApplication.translate
        WDark.setWindowTitle(_translate("WDark", "Frelon corrections"))
        self.label.setText(_translate("WDark", "Dark background"))
        self.label_2.setText(_translate("WDark", "Dark sample"))
        self.label_3.setText(_translate("WDark", "Spline"))
        self.label_4.setText(_translate("WDark", "Flood"))
        self.incidenceCheckBox.setText(_translate("WDark", "Incidence angle correction (good for PDF)"))

from . import resources_rc



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from PyQt5 import QtCore, QtWidgets, QtGui
from ..bcommon import default
from .fixedwidget import FixedWidget
from .ui.ui_wsopts import Ui_WSOpts


class WSOpts(QtWidgets.QDialog, Ui_WSOpts, FixedWidget):
    def __init__(self, parent):
        super().__init__(parent)
        self.values = {}
        self.setUI()
        self.on_buttonBox_accepted()

    def setUI(self):
        self.setupUi(self)
        self.spinThreads.setMaximum(default.MAX_THREADS)
        self.fixWindow()

    def loadSettings(self):
        s = QtCore.QSettings()
        self.checkThreads.setChecked(s.value('WSOpts/checkThreads', False, bool))
        self.checkBins.setChecked(s.value('WSOpts/checkBins', False, bool))
        self.checkPol.setChecked(s.value('WSOpts/checkPol', False, bool))
        self.checkDouble.setChecked(s.value('WSOpts/checkDouble', False, bool))
        self.checkHalina.setChecked(s.value('WSOpts/checkHalina', False, bool))
        self.checkSA.setChecked(s.value('WSOpts/checkSA', True, bool))
        self.spinPol.setValue(s.value('WSOpts/spinPol', 0, float))
        self.spinBins.setValue(s.value('WSOpts/spinBins', 1, int))
        self.spinThreads.setValue(s.value('WSOpts/spinThreads', 1, int))

    def saveSettings(self):
        s = QtCore.QSettings()
        s.setValue('WSOpts/checkThreads', self.checkThreads.isChecked())
        s.setValue('WSOpts/checkBins', self.checkBins.isChecked())
        s.setValue('WSOpts/checkPol', self.checkPol.isChecked())
        s.setValue('WSOpts/checkDouble', self.checkDouble.isChecked())
        s.setValue('WSOpts/checkHalina', self.checkHalina.isChecked())
        s.setValue('WSOpts/checkSA', self.checkSA.isChecked())
        s.setValue('WSOpts/spinPol', self.spinPol.value())
        s.setValue('WSOpts/spinBins', self.spinBins.value())
        s.setValue('WSOpts/spinThreads', self.spinThreads.value())

    def showEvent(self, event: QtGui.QShowEvent):
        super().showEvent(event)
        self.values = {}
        for name in self.__dict__:
            widget = self.__dict__[name]
            if isinstance(widget, QtWidgets.QAbstractSpinBox):
                self.values[name] = widget.value()
            elif isinstance(widget, QtWidgets.QCheckBox):
                self.values[name] = widget.isChecked()

    @QtCore.pyqtSlot(bool)
    def on_checkThreads_toggled(self, checked):
        self.spinThreads.setEnabled(checked)

    @QtCore.pyqtSlot(bool)
    def on_checkBins_toggled(self, checked):
        self.spinBins.setEnabled(checked)

    @QtCore.pyqtSlot(bool)
    def on_checkPol_toggled(self, checked):
        self.spinPol.setEnabled(checked)

    @QtCore.pyqtSlot()
    def on_buttonBox_accepted(self):
        self.params = {
            'polarization': self.spinPol.value() if self.checkPol.isChecked() else -2,
            'threads': self.spinThreads.value() if self.checkThreads.isChecked() else 0,
            'bins': self.spinBins.value() if self.checkBins.isChecked() else 0,
            'solidangle': self.checkSA.isChecked(),
            'halina': self.checkHalina.isChecked(),
            'double': self.checkDouble.isChecked(),
        }

    @QtCore.pyqtSlot()
    def on_buttonBox_rejected(self):
        for name in self.values:
            widget = self.__dict__[name]
            if isinstance(widget, QtWidgets.QAbstractSpinBox):
                widget.setValue(self.values[name])
            elif isinstance(widget, QtWidgets.QCheckBox):
                widget.setChecked(self.values[name])



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import subprocess
from setuptools import setup


package_name = 'bubble'
package_version = '2017.10.21'


frozen_name = 'bubble/frozen.py'
we_run_setup = False
if not os.path.exists(frozen_name):
    we_run_setup = True
    hash_ = subprocess.Popen(['hg', 'id', '-i'], stdout=subprocess.PIPE).stdout.read().decode().strip()
    print(f'Bubble mercurial hash is {hash_}')
    with open(frozen_name, 'w') as frozen:
        frozen.write(f'# -*- coding: utf-8 -*-\nhg_hash = "{hash_}"\nversion = "{package_version}"\n')


setup(
    name='bubble-dubble',
    version=package_version,
    description='Azimuthal powder integration',
    author='Vadim Dyadkin',
    author_email='dyadkin@gmail.com',
    url='https://hg.3lp.cx/bubble',
    license='GPLv3',
    install_requires=[
        'numpy>=1.9',
        'cryio>=2016.11.20',
        'integracio>=2016.12.14',
        'decor>=2017.5.12',
        'pyqtgraph>=0.10.0',
        'Pillow',
        'scipy',
    ],
    entry_points={
        'console_scripts': [
            f'bubbles={package_name}.bserver.__init__:main',
        ],
        'gui_scripts': [
            f'bubblec={package_name}.bclient.__init__:main',
        ],
    },
    packages=[
        'bubble',
        'bubble.bclient',
        'bubble.bclient.ui',
        'bubble.bcommon',
        'bubble.bserver',
    ],
)


if we_run_setup:
    os.remove(frozen_name)



#!/usr/bin/python
# -*- coding: utf-8 -*-

import json
from PyQt5 import QtCore, QtGui, QtWidgets
from .ui.wsaxsparams import Ui_WSaxsParams
from .fixedwidget import FixedWidget


class WSaxsParams(QtWidgets.QDialog, Ui_WSaxsParams, FixedWidget):
    def __init__(self, parent):
        super().__init__(parent)
        self.setupUi(self)
        self.concentrationLineEdit.setValidator(QtGui.QDoubleValidator())
        self.thicknessLineEdit.setValidator(QtGui.QDoubleValidator())
        self.calibrationLineEdit.setValidator(QtGui.QDoubleValidator())
        self.params = {
            'concentration': 0,
            'thickness': 1,
            'calibration': 1e6,
        }
        self.fixWindow()

    def showEvent(self, event):
        self.concentrationLineEdit.setText(str(self.params['concentration']))
        self.thicknessLineEdit.setText(str(self.params['thickness']))
        self.calibrationLineEdit.setText(str(self.params['calibration']))

    @QtCore.pyqtSlot()
    def on_buttonBox_accepted(self):
        self.params = {
            'concentration': float(self.concentrationLineEdit.text()),
            'thickness': float(self.thicknessLineEdit.text()),
            'calibration': float(self.calibrationLineEdit.text()),
        }

    def saveSettings(self):
        s = QtCore.QSettings()
        s.setValue('WSaxsParams/params', json.dumps(self.params))

    def loadSettings(self):
        s = QtCore.QSettings()
        params = s.value('WSaxsParams/params', '', str)
        if params:
            self.params = json.loads(params)



#!/usr/bin/python
# -*- coding: utf-8 -*-

import os
import json
import pickle
import struct
import time
import cryio
import decor
import numpy as np
from PyQt5 import QtCore, QtNetwork
from ..bcommon import compressor, default


class Connector(QtCore.QObject):
    serverStateSignal = QtCore.pyqtSignal(dict)
    errorsSignal = QtCore.pyqtSignal(str)
    warningsSignal = QtCore.pyqtSignal(str)
    waxsStateSignal = QtCore.pyqtSignal(dict)
    saxsStateSignal = QtCore.pyqtSignal(dict)
    waxsPlotSignal = QtCore.pyqtSignal(dict)
    saxsPlotSignal = QtCore.pyqtSignal(dict)

    def __init__(self):
        super().__init__()
        self.buffer = b''
        self.header = b''
        self.size = 0
        self.unpackWaxsChi = False
        self.unpackSaxsChi = False
        self.saxsTimestamp = 0
        self.waxsTimestamp = 0
        self.localServer = False
        self.beamline = 'Dubble'
        self.host = default.DEFAULT_HOST
        self.port = default.DEFAULT_PORT
        self.socketTimer = QtCore.QTimer(self)
        self.socket = QtNetwork.QTcpSocket(self)
        self.serverHasStopped()
        self.connectTimer = QtCore.QTimer(self)
        self.connectSignals()
        self.connectTimer.start(1000)

    # noinspection PyUnresolvedReferences
    def connectSignals(self):
        self.socketTimer.timeout.connect(self.sendStateRequest)
        self.socket.connected.connect(self.sendFirstRequest)
        self.socket.readyRead.connect(self.readResponse)
        self.socket.disconnected.connect(self.serverHasStopped)
        self.socket.error.connect(self.serverHasError)
        self.connectTimer.timeout.connect(self.tryToConnect)

    def tryToConnect(self):
        if self.socket.state() == QtNetwork.QTcpSocket.UnconnectedState:
            self.connectToServer(self.host, self.port)

    def tryServer(self, host, port):
        if host in ('127.0.0.1', 'localhost', 'localhost6', 'localhost.localdomain'):
            self.localServer = True
        self.connectTimer.start(1000)
        self.connectToServer(host, port)

    def connectToServer(self, host, port):
        self.host = host
        self.port = port
        self.socket.close()
        self.socket.connectToHost(host, port)

    def sendFirstRequest(self):
        self.send({'state': 1, 'runLocally': self.localServer})
        self.socketTimer.start(1000)

    def sendStateRequest(self):
        self.send(self.stateRequest())

    def readResponse(self):
        self.buffer += bytes(self.socket.readAll())
        while len(self.buffer) >= default.PICKLE_HEADER_SIZE:
            if not self.header:
                self.header = self.buffer[:default.PICKLE_HEADER_SIZE]
                self.buffer = self.buffer[default.PICKLE_HEADER_SIZE:]
                magic, self.size = struct.unpack(default.PICKLE_HEADER_STRUCT, self.header)
                if magic != default.PICKLE_MAGIC_NUMBER:
                    self.header = b''
                    continue
            if len(self.buffer) < self.size:
                break
            packet = self.buffer[:self.size]
            self.buffer = self.buffer[self.size:]
            self.header = b''
            response = pickle.loads(packet)
            self.parseResponse(response)

    def serverHasStopped(self):
        self.serverStateSignal.emit({'serverRunning': ''})
        self.saxsStateSignal.emit({'saxs': {'running': False}})
        self.waxsStateSignal.emit({'waxs': {'running': False}})
        self.socketTimer.stop()
        self.socket.close()

    def serverHasError(self):
        pass

    def send(self, obj):
        obj['pickle'] = True
        request = '{}\r\n'.format(json.dumps(obj))
        if self.socket.state() == self.socket.ConnectedState:
            self.socket.write(request.encode(errors='ignore'))

    def stateRequest(self):
        return {'state': 1}

    def killRequest(self):
        return {'kill': 1}

    def parseResponse(self, response):
        if not response:
            return
        self.parseServerInfo(response.get('server', {}))
        self.parseWaxsResponse(response.get('waxs', {}))
        self.parseSaxsResponse(response.get('saxs', {}))
        if not self.parseErrorsResponse(response.get('errors', {})):
            self.parseWarningsResponse(response.get('warnings', {}))

    def parseErrorsResponse(self, errors):
        if errors:
            self.errorsSignal.emit(self.joinServerMessages(errors))
            return True
        return False

    def parseWarningsResponse(self, warnings):
        if warnings:
            self.warningsSignal.emit(self.joinServerMessages(warnings))

    def joinServerMessages(self, messages):
        return '\n'.join('{:d}. {}'.format(i, s) for i, s in enumerate(messages, 1))

    def unpackResponseArrays(self, response):
        data1d = response.get('data1d', None)
        image = response.get('image', None)
        if data1d:
            response['data1d'] = [compressor.decompressNumpyArray(a, False) for a in data1d]
        if image:
            response['image'] = compressor.decompressNumpyArray(image, False)
        return response

    def parseWaxsResponse(self, response):
        timestamp = response.get('timestamp', 0)
        if timestamp > self.waxsTimestamp:
            self.waxsTimestamp = timestamp
            response = self.unpackResponseArrays(response)
            self.waxsPlotSignal.emit(response)
        self.waxsStateSignal.emit(response)

    def parseSaxsResponse(self, response):
        timestamp = response.get('timestamp', 0)
        if timestamp > self.saxsTimestamp:
            self.saxsTimestamp = timestamp
            response = self.unpackResponseArrays(response)
            self.saxsPlotSignal.emit(response)
        self.saxsStateSignal.emit(response)

    def parseServerInfo(self, serverinfo):
        serverRuntime = self.parseServerTimestamp(serverinfo.get('startedTimestamp', 0))
        serverKilled = serverinfo.get('killed', True)
        self.serverStateSignal.emit({'serverRunning': serverRuntime, 'killed': serverKilled})

    def parseServerTimestamp(self, timestamp):
        serverStartedTimestamp = int((time.time() - timestamp) / 60)
        s = 'Server is running for'
        if serverStartedTimestamp < 60:
            minutes = serverStartedTimestamp
            runTime = '{} {} minute{}'.format(s, minutes, 's' if minutes > 1 else '')
        elif 60 <= serverStartedTimestamp < 60 * 24:
            hours = serverStartedTimestamp // 60
            minutes = serverStartedTimestamp - hours * 60
            runTime = '{} {:d} hour{} and {:d} minute{}'.format(s, hours, 's' if hours > 1 else '', minutes,
                                                                's' if minutes > 1 else '')
        else:
            days = serverStartedTimestamp // (60 * 24)
            hours = serverStartedTimestamp // 60 - days * 24
            minutes = serverStartedTimestamp - hours * 60 - days * 24 * 60
            runTime = '{} {:d} day{}, {:d} hour{} and {:d} minute{}'.format(s, days, 's' if days > 1 else '', hours,
                                                                            's' if hours > 1 else '', minutes,
                                                                            's' if minutes > 1 else '')
        return runTime

    def disconnectFromServer(self):
        self.socket.close()
        self.socketTimer.stop()
        self.connectTimer.stop()

    def killServer(self):
        self.send(self.killRequest())

    def getMask(self, maskFile, itype):
        mask = maskFile
        if os.path.exists(maskFile):
            try:
                mask = cryio.openImage(maskFile)
            except (IOError, cryio.numpymask.NotNumpyMask, cryio.fit2dmask.NotFit2dMask):
                pass
            else:
                mask = decor.correctMask(self.beamline, itype, mask).array
                mask[mask == 1] = -1
        return mask

    def packMask(self, params, itype):
        mask = self.getMask(params.get('maskFile', ''), itype)
        if isinstance(mask, np.ndarray):
            params['maskFile'] = compressor.compressNumpyArray(mask)

    def packPoni(self, params):
        poni = params.get('poni', '')
        if os.path.exists(poni):
            params['poni'] = compressor.compressFile(poni)

    def packBackground(self, params):
        pass

    def runSaxsWaxs(self, params, itype):
        self.packPoni(params)
        self.packBackground(params)
        self.packMask(params, itype)
        self.send({itype: params})

    def setBeamline(self, beamline):
        self.beamline = beamline



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'bubble/bclient/ui/ui_wsopts.ui'
#
# Created by: PyQt5 UI code generator 5.8.2
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_WSOpts(object):
    def setupUi(self, WSOpts):
        WSOpts.setObjectName("WSOpts")
        WSOpts.resize(514, 377)
        WSOpts.setLocale(QtCore.QLocale(QtCore.QLocale.English, QtCore.QLocale.UnitedStates))
        self.gridLayout = QtWidgets.QGridLayout(WSOpts)
        self.gridLayout.setObjectName("gridLayout")
        self.checkSA = QtWidgets.QCheckBox(WSOpts)
        self.checkSA.setChecked(True)
        self.checkSA.setObjectName("checkSA")
        self.gridLayout.addWidget(self.checkSA, 0, 0, 1, 1)
        self.checkDouble = QtWidgets.QCheckBox(WSOpts)
        self.checkDouble.setObjectName("checkDouble")
        self.gridLayout.addWidget(self.checkDouble, 1, 0, 1, 1)
        self.checkHalina = QtWidgets.QCheckBox(WSOpts)
        self.checkHalina.setObjectName("checkHalina")
        self.gridLayout.addWidget(self.checkHalina, 2, 0, 1, 1)
        self.checkPol = QtWidgets.QCheckBox(WSOpts)
        self.checkPol.setObjectName("checkPol")
        self.gridLayout.addWidget(self.checkPol, 3, 0, 1, 1)
        self.spinPol = QtWidgets.QDoubleSpinBox(WSOpts)
        self.spinPol.setEnabled(False)
        self.spinPol.setMinimum(-1.0)
        self.spinPol.setMaximum(1.0)
        self.spinPol.setSingleStep(0.01)
        self.spinPol.setProperty("value", 0.99)
        self.spinPol.setObjectName("spinPol")
        self.gridLayout.addWidget(self.spinPol, 3, 1, 1, 1)
        self.checkThreads = QtWidgets.QCheckBox(WSOpts)
        self.checkThreads.setObjectName("checkThreads")
        self.gridLayout.addWidget(self.checkThreads, 4, 0, 1, 1)
        self.spinThreads = QtWidgets.QSpinBox(WSOpts)
        self.spinThreads.setEnabled(False)
        self.spinThreads.setMinimum(1)
        self.spinThreads.setMaximum(50)
        self.spinThreads.setObjectName("spinThreads")
        self.gridLayout.addWidget(self.spinThreads, 4, 1, 1, 1)
        self.checkBins = QtWidgets.QCheckBox(WSOpts)
        self.checkBins.setObjectName("checkBins")
        self.gridLayout.addWidget(self.checkBins, 5, 0, 1, 1)
        self.spinBins = QtWidgets.QSpinBox(WSOpts)
        self.spinBins.setEnabled(False)
        self.spinBins.setMinimum(1)
        self.spinBins.setMaximum(100000)
        self.spinBins.setObjectName("spinBins")
        self.gridLayout.addWidget(self.spinBins, 5, 1, 1, 1)
        self.buttonBox = QtWidgets.QDialogButtonBox(WSOpts)
        self.buttonBox.setOrientation(QtCore.Qt.Horizontal)
        self.buttonBox.setStandardButtons(QtWidgets.QDialogButtonBox.Cancel|QtWidgets.QDialogButtonBox.Ok)
        self.buttonBox.setObjectName("buttonBox")
        self.gridLayout.addWidget(self.buttonBox, 6, 0, 1, 2)

        self.retranslateUi(WSOpts)
        self.buttonBox.accepted.connect(WSOpts.accept)
        self.buttonBox.rejected.connect(WSOpts.reject)
        QtCore.QMetaObject.connectSlotsByName(WSOpts)

    def retranslateUi(self, WSOpts):
        _translate = QtCore.QCoreApplication.translate
        WSOpts.setWindowTitle(_translate("WSOpts", "Server options"))
        self.checkSA.setText(_translate("WSOpts", "Solid angle correction"))
        self.checkDouble.setText(_translate("WSOpts", "Double precision"))
        self.checkHalina.setText(_translate("WSOpts", "Halina\'s equation for SAXS"))
        self.checkPol.setText(_translate("WSOpts", "Polarization correction"))
        self.checkThreads.setText(_translate("WSOpts", "Number of threads"))
        self.checkBins.setText(_translate("WSOpts", "Number of bins"))




#!/usr/bin/python
# -*- coding: utf-8 -*-

import os
import sys
import time
import traceback
import numpy as np
from scipy.misc import imresize
import cryio
import decor
from ..bcommon import compressor


class Integrator:
    MAX_ATTEMPTS = 10

    def __call__(self, filename, interface, catchException=True):
        image = self.open_image(filename, interface.is_monitor)
        if not image:
            return None
        if catchException:
            # noinspection PyBroadException
            try:
                result = self._integrate(filename, image, interface)
            except Exception:
                traceback.print_exc(file=sys.stdout)
                print(f'Frame: {filename}')
                return None
            else:
                return result
        else:
            return self._integrate(filename, image, interface)

    def _integrate(self, filename, frames, interface):
        retval = None
        ext = interface.ext
        if ext.startswith('.'):
            ext = ext[1:]
        if frames.header.get('Bubble_normalized', 0):
            return retval
        path, base = os.path.split(filename)
        newpath = os.path.join(path, interface.subdir)
        if frames.multiframe:
            newpath = os.path.join(newpath, '.'.join(base.split('.')[:-1]))
        try:  # it seems that here we can have race conditions
            os.makedirs(newpath)
        except OSError:
            pass
        for i, image in enumerate(frames):
            image = self.apply_corrections(interface, image)
            res, chi = self._do_integration(interface, image, base, newpath, ext, i)
            self.saveNormalizedFrame(interface, chi, image)
            img, data = self.pack_data(interface, image, res)
            interface.counter += 1
            retval = {
                'imageFile': filename,
                'chiFile': chi,
                'transmission': image.transmission_coefficient,
                'data1d': data,
                'image': img,
                'timestamp': time.time(),
                'sent': False,
            }
        return retval

    def open_image(self, filename, is_monitor):
        i = 0
        while True:
            i += 1
            # noinspection PyBroadException
            try:
                image = cryio.openImage(filename)
                if is_monitor and isinstance(image, cryio.cbfimage.CbfImage) and 'Flux' not in image.header:
                    raise ValueError(f'Cbf file {filename} does not contain the "Flux" key in the header.\n'
                                     f'Either switch off the normalization by monitor or wait until all the headers '
                                     f'are written')
            except Exception:
                if i < self.MAX_ATTEMPTS:
                    print(f'File {filename} could not be read {i:d} times, another attempt in 1 sec...')
                    time.sleep(1)
                else:
                    traceback.print_exc(file=sys.stdout)
                    print(f'Frame {filename} could not be read, see exception above :(')
                    return None
            else:
                return image

    def apply_corrections(self, i, image):
        image.float()
        image = i.darksample(image)
        image = i.flatfield(image)
        image = i.normalize(image)
        image = i.background(image, i.calcTransmission, i.thickness, i.concentration, i.halina)
        image.array *= i.calibration
        image = i.mask(image)
        image.array = i.distortion(image.array)
        return image

    def pack_data(self, i, image, res):
        img = None
        rs = None
        if not i.speed:
            img = decor.correctImage(i.beamline, i.type, image.array)
            img = imresize(img, 40)
            img = compressor.compressNumpyArray(img, not i.pickle)
        if not i.sspeed:
            rs = [compressor.compressNumpyArray(array, not i.pickle) for array in res]
        return img, rs

    def _do_integration(self, i, image, base, newpath, ext, num):
        res, chi = None, None
        ext = ext if ext else 'dat'
        mf = '_{:05d}'.format(num) if image.multiframe else ''
        if i.azimuthChecked and i.azimuthSlices:
            step = i.azimuth[0]
            for aslice in np.arange(0, 360, step):
                azmin, azmax = aslice, aslice + step
                res = i.i(image.array, azmin, azmax)
                chi = '{}{}_{:03.0f}_{:03.0f}.{}'.format('.'.join(base.split('.')[:-1]), mf, azmin, azmax, ext)
                self._save_results(i, newpath, chi, res, image.transmission_coefficient)
        else:
            i.i.azimuth = i.azimuth if i.azimuthChecked else None
            res = i.i(image.array)
            chi = '{}{}.{}'.format('.'.join(base.split('.')[:-1]), mf, ext)
            res = self._save_results(i, newpath, chi, res, image.transmission_coefficient)
        return res, chi

    def correctIncidence(self, i, res):
        if i.detector == 'Frelon' and i.incidence:
            return decor.correctFrelonIncidence(i.wavelength, res)
        return res

    def _save_results(self, i, newpath, chi, res, transCoef):
        res = np.array(res)
        res = self.correctIncidence(i, res)
        res = i.normalize.norm_after(res, i.calibration)
        chi = os.path.join(newpath, chi)
        np.savetxt(chi, res.T, '%.6e')
        if i.calcTransmission:
            with open(chi, 'r+') as fchi:
                old = fchi.read()
                fchi.seek(0)
                fchi.write('# Transmission coefficient {0:.7f}\n{1}'.format(transCoef, old))
        return res

    def saveNormalizedFrame(self, i, chifile, image):
        if i.save:
            image.array = image.array.astype(np.int32)
            name = '{}_norm.edf'.format(chifile[:-4])
            image.save_edf(name, Bubble_normalized=1)
